import geometry as geomtr
from model_conf import *
import sqlite3
import pandas as pd
from datetime import datetime
import numpy as  np
from scipy import interpolate
from model_conf import input_data

db_path=input_data['db_path']

def observations_to_it2_PT(db_path=db_path,wells=input_data['WELLS'],T_DEV=input_data['IT2']['T_DEV'],P_DEV=input_data['IT2']['P_DEV']):
	"""Genera archivo "../model/it2/observations_PT.dat"

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf
	wells : array
	  Arreglo con nombre de pozos a ser incluidos, tomado de model_conf
	T_DEV  : float
	  Desviacion a considerar para las mediciones de temperatura, normalmente 5.0 [C]
	P_DEV : float
	  Desviacion a considerar para las mediciones de presion, sugerido 10 [bar]

	Returns
	-------
	file
	  observations_PT.dat: Archivo con observaciones de temperatura y presion en tipo 0 de corrida

	Attention
	---------
	La fuente de informacion es la base de datos sqlite

	Examples
	--------
	>>> observations_to_it2_PT(db_path,wells,5,10)
	"""
	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	q_layer="SELECT correlative,middle FROM layers"

	corr_layer=[]
	middle_tvd=[]

	c.execute(q_layer)
	rows=c.fetchall()
	for row in rows:
		corr_layer.append(row[0])
		middle_tvd.append(row[1])

	string=""
	string_T="""	>>TEMPERATURE"""
	string_P="""	>>PRESSURE"""

	for name in sorted(wells):

		q_corr="SELECT blockcorr FROM t2wellblock WHERE well='%s'"%(name)

		c.execute(q_corr)
		rows=c.fetchall()
		for row in rows:
			blockcorr=row[0]

		for tvdn in range(len(middle_tvd)):

			q_PT="SELECT MeasuredDepth, Pressure, Temperature from PT WHERE well='%s' ORDER BY MeasuredDepth"%name
			c.execute(q_PT)
			rows=c.fetchall()
			
			md=[]
			P_md=[]
			T_md=[]
			for row in rows:
				md.append(row[0])
				P_md.append(row[1])
				T_md.append(row[2])

			T_md= [np.nan if x == 0 else x for x in T_md]
			P_md= [np.nan if x == 0 else x for x in P_md]
			
			if not np.isnan(np.mean(T_md)):
				x_V,y_V,z_V,var_V=geomtr.MD_to_TVD_one_var_array(name,T_md,md,100)
				func_T=interpolate.interp1d(z_V,var_V)

			try:
				if func_T(middle_tvd[tvdn])>0:
					string_T+="""
			>>> ELEMENT : %s
				>>>> ANNOTATION : %s-T-%s
				>>>> DEVIATION  : %s
				>>>> WINDOW     : 0.0 60.0 [SECONDS]
				>>>> DATA
						 0 %s
				<<<<\n"""%(corr_layer[tvdn]+blockcorr,corr_layer[tvdn]+blockcorr,name,T_DEV,func_T(middle_tvd[tvdn]))
			except ValueError:
				pass

			if not np.isnan(np.mean(P_md)) and np.mean(P_md)>0:
				x_V,y_V,z_V,var_V=geomtr.MD_to_TVD_one_var_array(name,P_md,md,100)
				func_P=interpolate.interp1d(z_V,var_V)

				try:
					if func_P(middle_tvd[tvdn])>1.0:
						string_P+="""
			>>> ELEMENT : %s
				>>>> ANNOTATION : %s-P-%s
				>>>> FACTOR     : 1.0E5 [bar] - [Pa]
				>>>> DEVIATION  : %s
				>>>> WINDOW     : 0.0 60.0 [SECONDS]
				>>>> DATA
					    0 %s
				<<<<\n"""%(corr_layer[tvdn]+blockcorr,corr_layer[tvdn]+blockcorr,name,P_DEV,func_P(middle_tvd[tvdn]))
				except ValueError:
					pass

	string_T+="		<<<\n"
	string_P+="		<<<\n"
	string+=string_T
	string+=string_P
	observation_file=open("../model/it2/observations_PT.dat",'w')
	observation_file.write(string)
	observation_file.close()

def observations_to_it2_h(db_path=db_path,wells=input_data['WELLS'],h_DEV=input_data['IT2']['h_DEV']):
	"""Genera archivo "../model/it2/observations_h.dat"

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf
	wells : array
	  Arreglo con nombre de pozos a ser incluidos, tomado de model_conf
	h_DEV :float
	  Desviacion a considerar para las mediciones de entapia, sugerido 200 [kJ/kg]

	Returns
	-------
	file
	  observations_h.dat: Archivo con observaciones de entalpia a lo largo del tiempo

	Attention
	---------
	La fuente de informacion es la base de datos sqlite y referenciados al tiempo 0 (ref_date de model_conf)

	Examples
	--------
	>>> observations_to_it2_h(db_path,wells,200)
	"""
	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	ref_date=input_data['ref_date']

	string="	>>ENTHALPY\n"

	for name in sorted(wells):

		#X and Z is reserved to new well, which do not have a real flow or enthalpy
		q_source="SELECT source_nickname FROM t2wellsource WHERE well='%s'"%name
		c.execute(q_source)
		rows=c.fetchall()
		for row in rows:
			source_nickname=row[0]

		data=pd.read_sql_query("SELECT flowing_enthalpy, date_time,(steam_flow+liquid_flow) as flow FROM mh WHERE well='%s' ORDER BY date_time;"%name,conn)

		dates_func=lambda datesX: datetime.strptime(datesX, "%Y-%m-%d %H:%M:%S")
		#Read file cooling
		dates=list(map(dates_func,data['date_time']))

		if len(dates)>0:
			if (min(dates)-ref_date).total_seconds()<0:
				min_window=0
			else:
				min_window=(min(dates)-ref_date).total_seconds()
			string+="""		>>>SINK: %s
			>>>> ANNOTATION: %s-FLOWH
			>>>> FACTOR    : 1000 [kJ/kg] - [J/kg]
			>>>> DEVIATION : %s   [kJ/kg]
			>>>> WINDOW     : %s %s [SECONDS]
			>>>> DATA\n"""%(source_nickname,name,h_DEV,min_window,(max(dates)-ref_date).total_seconds())

			for n in range(len(dates)):
				timex=(dates[n]-ref_date).total_seconds()
				if data['flowing_enthalpy'][n]>0 and data['flow'][n]>0 :
					string_x="				 %s 	%6.3E\n"%(timex,data['flowing_enthalpy'][n])
					string+=string_x
			string+="			<<<<\n"
	string+="""		<<<\n"""

	observation_file_h=open("../model/it2/observations_h.dat",'w')
	observation_file_h.write(string)
	observation_file_h.close()

def observations_to_it2_DD(db_path=db_path,wells=input_data['WELLS'],P_DEV=input_data['IT2']['P_DEV'],include_pres=False,p_res_block=None):
	"""Genera archivo "../model/it2/observations_dd.dat"

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf
	wells : array
	  Arreglo con nombre de pozos a ser incluidos, tomado de model_conf
	P_DEV : float
	  Desviacion a considerar para las mediciones de presion, sugerido 10 [bar]

	Returns
	-------
	file
	  observations_dd.dat: Archivo con observaciones de caida de presion a lo largo del tiempo

	Attention
	---------
	La fuente de informacion es la base de datos sqlite y referenciados al tiempo 0 (ref_date de model_conf)

	Examples
	--------
	>>> observations_to_it2_DD(db_path,wells,10)
	"""

	conn=sqlite3.connect(db_path)
	c=conn.cursor()
	ref_date=input_data['ref_date']

	string="	>>DRAWDOWN"

	for name in sorted(wells):
		q0="SELECT DISTINCT TVD from drawdown WHERE well='%s'"%name
		c.execute(q0)
		rows=c.fetchall()
		for row in rows:
			q1="SELECT correlative FROM layers WHERE middle=%s"%row[0]
			c.execute(q1)
			rows_corr=c.fetchall()
			if len(rows_corr)>0:
				q2="SELECT blockcorr FROM t2wellblock WHERE well='%s'"%name
				c.execute(q2)
				rows_bkc=c.fetchall()
				for row2 in rows_bkc:
					block=rows_corr[0][0]+row2[0]

				data=pd.read_sql_query("SELECT date_time, pressure FROM drawdown WHERE well='%s' AND TVD=%s ORDER BY date_time;"%(name,row[0]),conn)
				
				dates_func=lambda datesX: datetime.strptime(datesX, "%Y-%m-%d %H:%M:%S")
				#Read file cooling
				dates=list(map(dates_func,data['date_time']))

				if len(dates)>0:
					string+="""
		>>> ELEMENT: %s
			>>>> ANNOTATION: %s-DD-%s
			>>>> FACTOR     : 1.0E5 [bar] - [Pa]
			>>>> DEVIATION  : %s [bar]
			>>>> DATA\n"""%(block,name,row[0],P_DEV)

					for n in range(len(dates)):
						timex=(dates[n]-ref_date).total_seconds()
						string_x="				 %s 	%6.3E\n"%(timex,data['pressure'][n])
						string+=string_x
					string+="""			<<<<\n"""
	

	#iTOUGH2 de presion de reservorio

	if include_pres:
		pres_data=pd.read_csv("../input/drawdown/p_res.csv",delimiter=';')
		dates_func_res=lambda datesX: datetime.strptime(datesX, "%d/%m/%Y")
		dates_res=list(map(dates_func_res,pres_data['date']))
		string+="""
			>>> ELEMENT: %s
				>>>> ANNOTATION: RES
				>>>> FACTOR     : 1.0E5 [bar] - [Pa]
				>>>> DEVIATION  : %s [bar]
				>>>> DATA\n"""%(p_res_block,P_DEV)
		for n in range(len(dates_res)):
			timex=(dates_res[n]-ref_date).total_seconds()
			string_x="				 %s 	%6.3E\n"%(timex,pres_data['pres'][n])
			string+=string_x
		string+="""			<<<<\n"""
	
	string+="""		<<<\n"""

	observation_file_dd=open("../model/it2/observations_dd.dat",'w')
	observation_file_dd.write(string)
	observation_file_dd.close()

def observations_to_it2(db_path=db_path,wells=input_data['WELLS'],T_DEV=input_data['IT2']['T_DEV'],P_DEV=input_data['IT2']['P_DEV'],h_DEV=input_data['IT2']['h_DEV'],typer=input_data['TYPE_RUN']):
	"""Genera bloque OBSERVATION

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf
	wells : array
	  Arreglo con nombre de pozos a ser incluidos, tomado de model_conf
	T_DEV  : float
	  Desviacion a considerar para las mediciones de temperatura, normalmente 5.0 [C]
	P_DEV : float
	  Desviacion a considerar para las mediciones de presion, sugerido 10 [bar]
	h_DEV :float
	  Desviacion a considerar para las mediciones de entapia, sugerido 200 [kJ/kg]
	typer :str
	  Tipo de corrida:  natural o prod

	Returns
	-------
	file
	  observations_PT.dat: Archivo con observaciones de temperatura y presion en tipo 0 de corrida
	file
	  observations_h.dat : archivo con entalpia de flujo a lo largo del tiempo
	file
	  observations_dd.dat : archivo con variacion de presion en elemento a lo largo del tiempo
	file
	  it2_ob_prod.dat : archivo que recopila observations_PT, observations_h y observations_dd en caso de selccionar corrida tipo 'prod'
	file
	  it2_ob_nat.dat : archivo que recopila observations_PT en caso de selccionar corrida tipo 'natural'


	Attention
	---------
	La fuente de informacion es la base de datos sqlite y referenciados al tiempo 0 (ref_date de model_conf)

	Examples
	--------
	>>> observations_to_it2(db_path,wells,5,10,200,'prod')
	"""
	if typer=='production':
		observations_to_it2_PT(db_path,wells,T_DEV,P_DEV)
		observations_to_it2_h(db_path,wells,h_DEV)
		observations_to_it2_DD(db_path,wells,P_DEV)
		filenames = ["../model/it2/observations_PT.dat","../model/it2/observations_h.dat","../model/it2/observations_dd.dat"]
		filename='it2_ob_prod'
	elif typer=='natural':
		observations_to_it2_PT(db_path,wells,T_DEV,P_DEV)
		filenames = ["../model/it2/observations_PT.dat"]
		filename='it2_ob_nat'
	with open("../model/it2/%s"%filename, 'w') as outfile:
		outfile.write(">OBSERVATION\n")
		for fname in filenames:
			with open(fname) as infile:
				for line in infile:
					outfile.write(line)
		outfile.write("""	<<\n""")
	outfile.close()

#Sostenibilidad Ahuachapan
#observations_to_it2()
#observations_to_it2_h()
#observations_to_it2_DD(include_pres=True,p_res_block='DA128')
#observations_to_it2_PT()


import  geometry as geometry
from model_conf import input_data
from formats import plot_conf_color,plot_conf_marker,formats_t2
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from datetime import datetime, timedelta
import matplotlib.dates as mdates
import sys
import matplotlib.gridspec as gridspec
from iapws import IAPWS97
import t2geores_functions as t2funcs


def plot_vertical_layer_distribution(show_fig,sav_fig):
	"""Regresa informacion relacionada a las capas

	Parameters
	----------
	layers : dictionary
	  Diccionario generado en model_conf
	z0_level : int
	  Elevacion de referencia de model_conf
	show_fig: bool
	  Muestra figura
	sav_fig: bool
	  Guarda figura

	Other Parameters
	----------------
	int
	  font_title_size: tamanio de letra de titulo
	int
	  fontsizey_layers: Tamanio de letra de etiqueta vertical [m.a.s.l.]
	int
	  fontsize_label: Tamanio de letra de etiqueta vertical [layer]

	Examples
	--------
	>>> plot_vertical_layer_distribution(layers,z0_level,show_fig=True,sav_fig=False)

	"""

	font_title_size=12
	fontsizey_layers=10
	fontsize_label=8

	layers_info=geometry.vertical_layers(input_data['LAYERS'],input_data['z_ref'])

	fig=plt.figure(figsize=(2.5, 8), dpi=100)

	ax=fig.add_subplot(111)
	y_layer_plot=layers_info['top']
	x_layer_plot=[100]*len(layers_info['top'])

	ax.plot(x_layer_plot,y_layer_plot,'-r',alpha=0)
	Depth_lims=[min(layers_info['bottom']),max(layers_info['top'])]

	ax.set_ylim(Depth_lims)
	ax.set_xlim([95,105])

	ax.set_ylabel('m.a.s.l.',fontsize = fontsizey_layers)
	ax.xaxis.tick_top()
	ax.tick_params(axis='x',which='both',length=0,labeltop=False)
	ax.title.set_position([0.5,1.05])
	ax.xaxis.set_label_coords(0.5,1.15)
	ax.yaxis.set_label_coords(-0.2,0.5)
	ax.tick_params(axis='y', which='major', labelsize=fontsize_label,pad=1)

	#Set layers

	ax2 = ax.twinx()            
	ax2.set_yticks(layers_info['top'], minor=True)
	ax2.yaxis.grid(True, which='minor',linestyle='--', color='grey', alpha=0.6)
	ax2.set_yticks( layers_info['middle'], minor=False)
	ax2.set_yticklabels(layers_info['name'],fontsize=fontsize_label)
	ax2.tick_params(axis='y',which='both',length=0)
	ax2.set_ylabel('Layers',fontsize = fontsizey_layers)
	ax2.yaxis.set_label_coords(1.15,0.5)
	ax2.set_ylim(ax.get_ylim())
	plt.tight_layout()

	if show_fig:
		plt.show()
	if sav_fig:
		fig.savefig("../report/images/vertical_distribution.png", format='png')

#plot_vertical_layer_distribution(show_fig=True,sav_fig=False)

def permeability_plot(IRP,slr,sgr,show_fig,sav_fig):
	"""Genera imagen de distribucion de permeabilidades relativas. La imagen es almacenada en formato .svg en la direccion ../report/images/coreys_permeabilities.svg

	Parameters
	----------
	show_fig: bool
	  Muestra figura
	sav_fig: bool
	  Guarda figura
	slr: float
	  Saturacion de liquido, tomado de model_conf
	sgr: float
	  Saturacion de gas, tomado de model_conf
	IRP: int
	  Tipo de distribucion de permeabilidades relativas, actualmente solo 'corey'

	Examples
	--------
	>>> permeability_plot(IRP,slr,sgr,show_fig,sav_fig)
	"""

	fontsize_=26
	if IRP==3:

		krl=[]
		krg=[]
		saturation=[]
		plt.rcParams["font.family"] = "Times New Roman"

		for n in np.linspace(0,1,200):
			S=(n-slr)/(1-slr-sgr)
			to_krl=S**4
			to_krg=((1-S)**2)*(1-S**2)
			if to_krl>1:
				to_krl=1
			if to_krg>1:
				to_krg=1

			if n<slr and to_krg>0:
				to_krl=0

			if n<slr and to_krl>1:
				to_krl=1

			krl.append(to_krl)
			krg.append(to_krg)
			saturation.append(n)

		fig= plt.figure(figsize=(10, 10))
		ax1=fig.add_subplot(111)
		ax1.plot(saturation,krl,'--k',label='Liquid')
		ax1.plot(saturation,krg,'-k',label='Vapor')
		ax1.set_title("Corey RPF",fontsize=fontsize_)
		ax1.set_xlabel("Water saturation",fontsize=fontsize_)
		ax1.set_ylabel('Relative permeabilty',fontsize=fontsize_)
		ax1.set_ylim([0,1])
		ax1.set_xlim([0,1])
		ax1.legend(loc='lower left',frameon=False,fontsize=fontsize_)
		ax1.tick_params(axis='both', which='major', labelsize=fontsize_,pad=1)
		yticks = ax1.yaxis.get_major_ticks() 
		yticks[0].label1.set_visible(False)

		if show_fig:
			plt.show()

		if sav_fig:
			fig.savefig("../report/images/coreys_permeabilities.svg", format='svg') 

#permeability_plot(input_data['RPCAP']['IRP'],input_data['RPCAP']['RP1'],input_data['RPCAP']['RP2'],show_fig=True,sav_fig=False)

def plot_one_drawdown_from_txt(well,depth):
	"""Genera una grafica utilizando la informacion almacenada en ../input/drawdown para el pozo solicitado

	Parameters
	----------
	well : str
	  Pozo seleccionado
	depth : float
	  Profundidad vertical de grafico solicitado

	Returns
	-------
	graph
	  Presion vs tiempo para una profundidad especifica
	  
	Attention
	---------
	El archivo input/drawdown/{pozo}_DD.dat debe existir

	Note
	----
	La profundidad seleccionada debe estar en registrada en el archivo .dat

	Examples
	--------
	>>> plot_one_drawdown_from_txt('AH-34',0)
	"""

	#Read file
	file="../input/drawdown/%s_DD.dat"%well
	data=pd.read_csv("../input/drawdown/%s_DD.dat"%well)
	fontsize_layout=8

	if len(data.loc[data['TVD']==depth]['datetime'])>1:

		dates_func=lambda datesX: datetime.strptime(datesX, "%Y-%m-%d %H:%M:%S")
		dates=list(map(dates_func,data.loc[data['TVD']==depth]['datetime'].values))

		#Plotting

		fig, ax = plt.subplots(figsize=(10,4))
		ax.plot(dates,data.loc[data['TVD']==depth]['pressure'].values,linestyle='None',color=plot_conf_color['P'][0],marker=plot_conf_marker['real'][0],linewidth=1,ms=3,label='drawdown')
		ax.set_title("Drawdown at well: %s at %s masl"%(well,depth) ,fontsize=fontsize_layout)
		ax.set_xlabel("Time",fontsize = fontsize_layout)
		ax.set_ylabel('Pressure [bar]',fontsize = fontsize_layout)


		#Plotting formating
		xlims=[min(dates)-timedelta(days=365),max(dates)+timedelta(days=365)]
		ax.format_xdata = mdates.DateFormatter('%Y%-m-%d %H:%M:%S')

		years = mdates.YearLocator()
		years_fmt = mdates.DateFormatter('%Y')

		ax.set_xlim(xlims)
		ax.xaxis.set_major_formatter(years_fmt)

		#ax.xaxis.set_major_locator(years)
		#fig.autofmt_xdate()

		#Grid style
		ax.yaxis.grid(True, which='major',linestyle='--', color='grey', alpha=0.6)
		ax.xaxis.grid(True, which='major',linestyle='--', color='grey', alpha=0.6)
		ax.grid(True)
	
	else:
		sys.exit("""Error message: there is no drawdown data at %s for the well %s"""%(depth,well))
	return plt.show()

#plot_one_drawdown_from_txt('AH-34',0)

def plot_one_cooling_from_txt(well,depth):
	"""Genera una grafica utilizando la informacion almacenada en ../input/cooling para el pozo solicitado

	Parameters
	----------
	well : str
	  Pozo seleccionado
	depth : float
	  Profundidad de grafico solicitado

	Returns
	-------
	graph
	  Temperatura vs tiempo para una profundidad especifica
	  
	Attention
	---------
	El archivo input/cooling/{pozo}_C.dat debe existir

	Note
	----
	La profundidad seleccionada debe estar en registrada en el archivo .dat

	Examples
	--------
	>>> plot_one_cooling_from_txt('AH-34',0)
	"""

	#Read file
	file="../input/drawdown/%s_C.dat"%well
	data=pd.read_csv("../input/cooling/%s_C.dat"%well)
	data.loc[data['TVD']==depth]['datetime']
	dates_func=lambda datesX: datetime.strptime(datesX, "%Y-%m-%d %H:%M:%S")
	dates=list(map(dates_func,data.loc[data['TVD']==depth]['datetime'].values))

	fontsize_layout=8

	#Plotting

	if len(data.loc[data['TVD']==depth]['datetime'])>1:
		fig, ax = plt.subplots(figsize=(10,4))
		ax.plot(dates,data.loc[data['TVD']==depth]['temperature'].values,linestyle='None',color=plot_conf_color['T'][0],marker=plot_conf_marker['real'][0],linewidth=1,ms=3,label='cooling')
		ax.set_title("Well: %s at %s masl"%(well,depth) ,fontsize=fontsize_layout)
		ax.set_xlabel("Time",fontsize = fontsize_layout)
		ax.set_ylabel('Temperature [$^\circ$C]',fontsize = fontsize_layout)

		#Plotting formating
		xlims=[min(dates)-timedelta(days=365),max(dates)+timedelta(days=365)]
		ax.format_xdata = mdates.DateFormatter('%Y%-m-%d %H:%M:%S')

		years = mdates.YearLocator()
		years_fmt = mdates.DateFormatter('%Y')

		ax.set_xlim(xlims)
		ax.xaxis.set_major_formatter(years_fmt)

		#Grid style
		ax.yaxis.grid(True, which='major',linestyle='--', color='grey', alpha=0.6)
		ax.xaxis.grid(True, which='major',linestyle='--', color='grey', alpha=0.6)
		ax.grid(True)
	else:
		sys.exit("""Error message: there is no cooling data at %s for the well %s"""%(depth,well))	

	return plt.show()

#plot_one_cooling_from_txt('AH-34',0)

def plot_one_cooling_and_drawdown_from_txt(well,depth):
	"""Genera una grafica utilizando la informacion almacenada en ../input/cooling y ../input/cooling para el pozo solicitado

	Parameters
	----------
	well : str
	  Pozo seleccionado
	depth : float
	  Profundidad de grafico solicitado

	Returns
	-------
	graph
	  Temperatura (eje1), presion (eje2) vs tiempo para una profundidad especifica
	  
	Attention
	---------
	El archivo ../input/drawdown/{pozo}_DD.dat e input/cooling/{pozo}_C.dat debe existir

	Note
	----
	La profundidad seleccionada debe estar en registrada en el archivo .dat

	Examples
	--------
	>>> plot_one_cooling_and_drawdown_from_txt('AH-34',0)
	"""
	dates_func=lambda datesX: datetime.strptime(datesX, "%Y-%m-%d %H:%M:%S")

	#Read file cooling
	data_c=pd.read_csv("../input/cooling/%s_C.dat"%well)
	dates_c=list(map(dates_func,data_c.loc[data_c['TVD']==depth]['datetime'].values))

	#Read file DRAWDOWN
	data_d=pd.read_csv("../input/drawdown/%s_DD.dat"%well)
	dates_d=list(map(dates_func,data_d.loc[data_d['TVD']==depth]['datetime'].values))
	if len(dates_d)==0:
		#print("There is not values for this depth")
		print("There is not values for this depth")
	else:
		#Plotting

		fig, ax = plt.subplots(figsize=(10,4))
		ax2 = ax.twinx()

		#Plotting drawdown
		ax.plot(dates_d,data_d.loc[data_d['TVD']==depth]['pressure'].values,linestyle='None',color=plot_conf_color['P'][0],marker=plot_conf_marker['real'][0],linewidth=1,ms=3,label='drawdown',alpha=0.75)
		ax.set_title("Well: %s at %s masl"%(well,depth) ,fontsize=8)
		ax.set_xlabel("Time",fontsize = 8)
		ax.set_ylabel('Pressure [bar]',fontsize = 8)

		#Plotting cooling
		ax2.plot(dates_c,data_c.loc[data_c['TVD']==depth]['temperature'].values,linestyle='None',color=plot_conf_color['T'][0],marker=plot_conf_marker['real'][0],linewidth=1,ms=3,label='cooling',alpha=0.75)
		ax2.set_ylabel('Temperature [$^\circ$C]',fontsize = 8)


		#Plotting formating
		xlims=[min(dates_d)-timedelta(days=365),max(dates_d)+timedelta(days=365)]
		ax.format_xdata = mdates.DateFormatter('%Y%-m-%d %H:%M:%S')

		years = mdates.YearLocator()
		years_fmt = mdates.DateFormatter('%Y')

		ax.set_xlim(xlims)
		ax.xaxis.set_major_formatter(years_fmt)

		fig.legend(bbox_to_anchor=(0.3, 0.26))

		#ax.xaxis.set_major_locator(years)
		#fig.autofmt_xdate()

		#Grid style
		ax.yaxis.grid(True, which='major',linestyle='--', color='grey', alpha=0.6)
		ax.xaxis.grid(True, which='major',linestyle='--', color='grey', alpha=0.6)
		ax.grid(True)
		
		return plt.show()

#plot_one_cooling_and_drawdown_from_txt('AH-34',0)

def plot_one_mh_from_txt(well):
	"""Genera una grafica utilizando la informacion almacenada en ../input/mh

	Parameters
	----------
	well : str
	  Pozo seleccionado

	Returns
	-------
	graph
	  Entalpia (eje1), flujo (eje2) vs tiempo 
	  
	Attention
	---------
	El archivo ../input/mh/{pozo}_mh.dat debe existir


	Examples
	--------
	>>> plot_one_mh_from_txt('AH-16A')
	"""


	fontsize_ylabel=8
	fontsize_title=9

	dates_func=lambda datesX: datetime.strptime(datesX, "%Y-%m-%d %H:%M:%S")

	#Read file cooling
	data=pd.read_csv("../input/mh/%s_mh.dat"%well)
	dates=list(map(dates_func,data['date-time'].values))
	data = data.replace(0,np.nan)


	#Setting plot
	gs = gridspec.GridSpec(3, 1)
	fig, ax = plt.subplots(figsize=(10,4))


	#Flow plot
	ax.format_xdata = mdates.DateFormatter('%Y%-m-%d %H:%M:%S')
	ax=plt.subplot(gs[0,0])
	ln1=ax.plot(dates,data['steam'],linestyle='-',color=plot_conf_color['ms'][0],marker=plot_conf_marker['real'][0],linewidth=1,ms=1,label='Steam',alpha=0.75)

	ax1b = ax.twinx()
	ln2=ax1b.plot(dates,data['liquid'],linestyle='-',color=plot_conf_color['ml'][0],marker=plot_conf_marker['real'][0],linewidth=1,ms=1,label='Liquid',alpha=0.75)
	ax.set_ylabel('Flow s[kg/s]',fontsize = fontsize_ylabel)
	ax1b.set_ylabel('Flow l[kg/s]',fontsize = fontsize_ylabel)
	
	# legend for flow
	lns = ln1+ln2
	labs = [l.get_label() for l in lns]
	ax.legend(lns, labs, loc="upper right")

	#Enthalpy plot
	ax2=plt.subplot(gs[1,0])
	ax2.plot(dates,data['enthalpy'],linestyle='-',color=plot_conf_color['h'][0],marker=plot_conf_marker['real'][0],linewidth=1,ms=1,label='Enthalpy',alpha=0.75)
	ax2.legend(loc="upper right")
	ax2.set_ylabel('Enthalpy [kJ/kg]',fontsize = fontsize_ylabel)


	#WHPressure plot
	ax3=plt.subplot(gs[2,0])
	ax3.plot(dates,data['WHPabs'],linestyle='-',color=plot_conf_color['P'][0],marker=plot_conf_marker['real'][0],linewidth=1,ms=1,label='Pressure',alpha=0.75)
	ax3.legend(loc="upper right")
	ax3.set_ylabel('Pressure [bara]',fontsize = fontsize_ylabel)

	years = mdates.YearLocator()
	years_fmt = mdates.DateFormatter('%Y')

	plt.setp(ax.get_xticklabels(), visible=False)
	plt.setp(ax2.get_xticklabels(), visible=False)

	ax.xaxis.set_major_formatter(years_fmt)
	ax2.xaxis.set_major_formatter(years_fmt)
	ax3.xaxis.set_major_formatter(years_fmt)

	fig.suptitle('Production history of well %s'%well,fontsize=fontsize_title)

	plt.show()

#plot_one_mh_from_txt('AH-16A')

def check_layers_and_feedzones(show_fig,sav_fig,source_txt = '../input/'):
	"""Regresa informacion relacionada a las capas

	Parameters
	----------
	show_fig: bool
	  Muestra figura
	sav_fig: bool
	  Guarda figura
	source_txt: str
	  Indica el directorio donde se encuentran los archivos de entrada

	Other Parameters
	----------------
	int
	  font_title_size: tamanio de letra de titulo
	int
	  fontsizey_layers: Tamanio de letra de etiqueta vertical [m.a.s.l.]
	int
	  fontsize_label: Tamanio de letra de etiqueta vertical [layer]

	Examples
	--------
	>>> plot_vertical_layer_distribution(layers,z0_level,show_fig,sav_fig=True)

	"""

	font_title_size=12
	fontsizey_layers=8
	fontsize_label=8

	layers_info=geometry.vertical_layers(input_data['LAYERS'],input_data['z_ref'])


	#Plot setting 
	fig=plt.figure(figsize=(20, 10), dpi=100)
	ax=fig.add_subplot(111)

	feedzones=pd.read_csv(source_txt+'well_feedzone_xyz.csv',delimiter=",")

	index2plot=[]
	wells_ticks=[]
	z_wells=[]
	for index,row in feedzones.iterrows():
		index2plot.append(index*4)
		wells_ticks.append(row['well'])
		z_wells.append(row['z'])
		ax.plot(5*[index*4],np.linspace(z0_level,row['z'],5),'-k',lw=1,alpha=0.5)
		#ax.text(index*4,row['z'],row['well'],fontsize=6)
	
	#Set feedzones position 

	ax.plot(index2plot,z_wells,'ok',ms=2)

	ax.set_xticks(index2plot)
	ax.set_xticklabels(wells_ticks,rotation=90)

	y_layer_plot=layers_info['top']
	x_layer_plot=[index]*len(layers_info['top'])

	ax.xaxis.tick_top()
	ax.tick_params(axis='x',which='both',labelsize=6)
	
	Depth_lims=[min(layers_info['bottom']),max(layers_info['top'])]

	ax.set_ylim(Depth_lims)
	ax.set_ylabel('m.a.s.l.',fontsize = fontsizey_layers)
	ax.set_yticks(layers_info['top'], minor=False)
	
	ax.title.set_position([0.5,1.05])
	ax.xaxis.set_label_coords(0.5,1.15)
	ax.yaxis.set_label_coords(-0.05,0.5)
	ax.tick_params(axis='y', which='major', labelsize=fontsizey_layers,pad=1)

	#Set layers

	ax2 = ax.twinx()            
	ax2.set_yticks(layers_info['top'], minor=True)
	ax2.yaxis.grid(True, which='minor',linestyle='--', color='grey', alpha=0.6)
	ax2.set_yticks(layers_info['middle'], minor=False)
	ax2.set_yticklabels(layers_info['name'],fontsize=fontsize_label)
	ax2.tick_params(axis='y',which='both',length=0)
	ax2.set_ylabel('Layer',fontsize = fontsizey_layers)
	ax2.yaxis.set_label_coords(1.05,0.5)
	ax2.set_ylim(ax.get_ylim())

	plt.tight_layout()

	if show_fig:
		plt.show()
	if sav_fig:
		fig.savefig("../report/images/vertical_distribution_wells.png", format='png') 

#check_layers_and_feedzones(show_fig=True,sav_fig=False)

def plot_init_conditions():

	T, P, depths =t2funcs.initial_conditions()

	fig= plt.figure(figsize=(8, 8), dpi=300)

	axT=fig.add_subplot(121)

	axP=fig.add_subplot(122)

	ln1T=axT.plot(T,depths,linestyle='-',color=plot_conf_color['T'][0],marker=plot_conf_marker['current'][0],linewidth=0.5,ms=1)

	ln1P=axP.plot(P,depths,linestyle='-',color=plot_conf_color['P'][0],marker=plot_conf_marker['current'][0],linewidth=0.5,ms=1)

	#Plot configuration

	fontsizex=8
	fontsize_axis=4
	fontsize_tick=3
	fontsize_legend=2

	fig.suptitle('Initial conditions', fontsize=fontsizex)

	axT.xaxis.tick_top()
	axT.set_xlabel('Temperature [$^\circ$C]',fontsize=fontsize_axis)
	axT.xaxis.set_label_coords(0.5,1.08)
	axT.tick_params(axis='both', which='major', labelsize=fontsize_tick,pad=1)
	axT.set_ylabel('TVD [m]',fontsize = fontsize_tick)
	axT.tick_params(axis='y',which='major',length=1)
	axT.set_xlim([0,max(T)])
	axT.set_ylim([min(depths),max(depths)])

	axP.xaxis.tick_top()
	axP.set_xlabel('Pressure [bar]',fontsize=fontsize_axis)
	axP.xaxis.set_label_coords(0.5,1.08)
	axP.tick_params(axis='both', which='major', labelsize=fontsize_tick,pad=1)
	axP.set_ylabel('TVD [m]',fontsize = fontsize_tick)
	axP.tick_params(axis='y',which='major',length=1)
	axP.set_xlim([0,max(P)])
	axP.set_ylim([min(depths),max(depths)])

	plt.show()

#plot_init_conditions()


import numpy as np
import re as re
import subprocess
import datetime
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import shutil
import os
import itertools
import json
import shapefile
import pylab as plb
import math
import sys
from scipy.spatial import ConvexHull
from model_conf import input_data, mesh_setup,geners
import pandas as pd
from matplotlib.ticker import (MultipleLocator, FormatStrFormatter,
                               AutoMinorLocator)
import pyvista as pv
import vtk
import  geometry as geometry

class py2amesh:
	"""Genera malla a partir de posiciones de pozos y malla regular

	Las principales caracteristicas de este clase son:

	-Genera una malla regular cuadrada o rectangular segun se necesite en toda la extension especificada.

	-En el interior (campo cercano) genera una malla en forma hexagonal delimitada por una extension cuadrada/rectangular o por un poligo definido a traves de un shapefile.

	-Permite la introduccion de lineas a lo largo de la malla, los cuales pueden representar fallas o limites.

	-La malla es generada por amesh

	-La nomenclatura es la siguiente: se utiliza el alfabeto para definir el nivel de la capa [A-Z], se utiliza como correlativo el alfabeto seguido por tres numeros, estos no deben contener dos ceros consecutivos o en el centro, por lo que el primero elemento sera AA110 y al llegar a AA199 saltara a AA210.

	-Los pozos definidos en el parametro <i>filename</i> son los primeros de la lista

	-Genera archivos de entrada para Steinar (RockEditor)

	-Permite almacenar una malla en formato shapefile

	-Permite graficar una capa seleccionada a traves de matplotlib (Para capas con muchos elementos se recomienda extrer formato shapefile)

	-Se generan dos archivos en formato json, conteniendo el bloque correlativo de cada pozo y otro con todos los bloques e informacion espacial de cada pozo.

	Parameters
	----------
	filename : str
		Nombre de archivo con ubicaciones de pozos (zonas de alimentacion)
	filepath : str
		Direccion donde se encuentran los archivos de entrada en general "../input"
	Xmin  : float
		Posicion X minima de malla
	Xmax : float
		Posicion X maxima de malla
	Ymin : float
		Posicion Y minima de malla
	Ymax : float
		Posicion Y maxima de malla
	toler : float
		Parametro Amesh para creacion de malla
	layers : dictionary
		Nombre y espesor de capas
	layer_to_plot : int
		En caso de graficar una capa utilizando matplotlib define la capa que se graficara
	x_space : float
		Distanciamiento horizontal de elementos en direccion X
	y_space : float
		Distanciamiento horizontal de elementos en direccion Y
	radius_criteria: float
		Distancia minima entre pozos y elementos de malla regular
	x_from_boarder: float
		Distancia horizontal donde se situara el primer elemento desde el borde
	y_from_boarder: float
		Distancia vertical donde se situara el primer elemento desde el borde
	x_gap_min: float
		Posicion X minima de malla de campo cercano
	x_gap_max: float
		Posicion X maxima de malla de campo cercano
	x_gap_space: float
		Distanciamiento horizontal de elementos en direccion X en campo cercano
	y_gap_min: float
		Posicion Y minima de malla de campo cercano
	y_gap_max: float
		Posicion Y maxima de malla de campo cercano
	y_gap_space: float
		Distanciamiento vertical de elementos en direccion X en campo cercano
	plot_names: bool
		En caso de generar grafico utiliznado matplotlib activa la representacion de los nombres de los bloques [0,1]
	plot_centers: bool
		En caso de generar grafico utiliznado matplotlib activa la representacion centros de los bloques con un punto [0,1]
	z0_level: float
		Nivel de referencia para capas
	mesh_creation: bool
		Ejecuta creacion de malla
	plot_layer: bool
		Genera grafico de capa seleccionada
	to_steinar: bool
		Genera archivos de entrada para RockEditor steinar
	to_GIS: bool
		Genera un archivo shapefile de la capa seleccionada
	plot_all_GIS: bool
		Genera un archivo shapefile para todas las capas
	from_leapfrog: bool
		lee archivos leapfrong ../mesh/from_leapfrog/LF_geometry.dat y ../mesh/from_leapfrog/LF_t2.dat, sin embargo se pierde la simbologia usada en leapfrog y no te utiliza la malla regular ni los pozos. Solamente se crea la malla utilizando amesh
	line_file: str
		En el caso de querer especificar una linea o curva por la cual se quieran generar elementos, especificar punto xy con las cabezeras: ID, X, Y en formato csv. Donde ID corresponde a un mismo elementos, por lo que para formar una linea se deber tener al menos un ID y dos pares xy
	fault_distance: float
		En caso de definir line_file, se crearan elementos paralelos a una distancia fault_distance a lo largo de los elementos
	with_polygon: bool
		De ser cierto leera un archivo shapefile dentro del cual se construira el campo cercano
	polygon_shape: str
		Archivo que delimita la geometria del campo cercano, este no debe contener concavidades internas.
	set_inac_from_poly: bool
		Establece como elementos inactivos a los que se encuentren afuera del poligono definido fuera del shapefile de entrada
	set_inac_from_inner:bool
		Establece como elementos inactivos a los que se encuentren afuera del poligono del campo cerano

	Returns
	-------
	file
	  eleme: elementos de malla especificados en formato TOUGH2, tanto en el formato AMESH como para lectura de steinar
	file
	  conne : conexiones de elementos de malla especificados en formato TOUGH2, tanto en el formato AMESH como para lectura de steinar
	shapefile
	  mesh_{field}_layer_{layer} : archivo shapefile de una o todas las capas, especialmente util cuando ya se ha definido la distribucion de roca.
	graph
	  En caso de ser especificados
	  
	Attention
	---------
	El ejecutable amesh debe estar en la carpeta scripts o en el directorio


	"""

	def __init__(self,filename,filepath,Xmin,Xmax,Ymin,Ymax,\
		toler,layers,layer_to_plot,x_space,y_space,radius_criteria,\
		x_from_boarder,y_from_boarder,\
		x_gap_min,x_gap_max,x_gap_space,y_gap_min,y_gap_max,y_gap_space,\
		plot_names,plot_centers,z0_level,plot_all_GIS,from_leapfrog,line_file,fault_distance,with_polygon,polygon_shape,set_inac_from_poly,set_inac_from_inner,rotate,angle,inner_mesh_type):
		self.filename=filename
		self.filepath=filepath
		self.layers=layers
		self.number_of_layer=len(layers)
		self.Xmin=Xmin
		self.Xmax=Xmax
		self.Ymin=Ymin
		self.Ymax=Ymax
		self.z0_level=z0_level

		self.layer_to_plot=layer_to_plot

		self.radius_criteria=radius_criteria

		self.x_space=x_space
		self.y_space=y_space

		self.x_from_boarder=x_from_boarder
		self.y_from_boarder=y_from_boarder

		self.z=0
		self.delf_rock="101"  #Layer index
		self.filename_out="in"
		self.toler=toler

		self.x_gap_min=x_gap_min
		self.x_gap_max=x_gap_max
		self.x_gap_space=x_gap_space
		self.y_gap_min=y_gap_min
		self.y_gap_max=y_gap_max
		self.y_gap_space=y_gap_space

		self.plot_names=plot_names
		self.plot_centers=plot_centers
		self.plot_all_GIS=plot_all_GIS
		self.from_leapfrog=from_leapfrog
		self.line_file=line_file
		self.fault_distance=fault_distance

		self.with_polygon=with_polygon
		self.set_inac_from_poly=set_inac_from_poly
		self.set_inac_from_inner=set_inac_from_inner
		self.rotate=rotate
		self.angle=angle
		self.inner_mesh_type=inner_mesh_type

		if self.with_polygon:
			shape = shapefile.Reader(polygon_shape)

			#first feature of the shapefile
			feature = shape.shapeRecords()[0]
			points = feature.shape.__geo_interface__ 
			self.polygon=[]
			for n in points:
				for v in points[n]:
					if n=='coordinates':
						self.polygon.append([v[0],v[1]]) # (GeoJSON format)

		self.color_dict = {1:[['AA','AB','AC','AD','AE','AF','AG'],'ROCK1','red'],\
						   2:[['BA','BB','BC','BD','BE','BF','BG'],'ROCK2','white'],\
						   3:[['CA','CB','CC','CD','CE','CF','CG'],'ROCK3','yellow'],\
						   4:[['DA','DB','DC','DD','DE','DF','DG'],'ROCK4','blue'],\
						   5:[['EA','EB','EC','ED','EE','EF','EG'],'ROCK5','green'],\
						   6:[['FA','FB','FC','FD','FE','FF','FG'],'ROCK6','purple'],\
						   7:[['GA','GB','GC','GD','GE','GF','GG'],'ROCK7','#ff69b4'],\
						   8:[['HA','HB','HC','HD','HE','HF','HG'],'ROCK8','darkorange'],\
						   9:[['IA','IB','IC','ID','IE','IF','IG'],'ROCK9','cyan'],\
						   10:[['JA','JB','JC','JD','JE','JF','JG'],'ROK10','magenta'],\
						   11:[['KA','KB','KC','KD','KE','KF','KG'],'ROK11','#faebd7'],\
						   12:[['LA','LB','LC','LD','LE','LF','LG'],'ROK12','#2e8b57'],\
						   13:[['MA','MB','MC','MD','ME','MF','MG'],'ROK13','#eeefff'],\
						   14:[['NA','NB','NC','ND','NE','NF','NG'],'ROK14','#da70d6'],\
						   15:[['OA','OB','OC','OD','OE','OF','OG'],'ROK15','#ff7f50'],\
						   16:[['PA','PB','PC','PD','PE','PF','PG'],'ROK16','#cd853f'],\
						   17:[['QA','QB','QC','QD','QE','QF','QG'],'ROK17','#bc8f8f'],\
						   18:[['RA','RB','RC','RD','RE','RF','RG'],'ROK18','#5f9ea0'],\
						   19:[['SA','SB','SC','SD','SE','SF','SG'],'ROK19','#daa520']}

		self.rock_dict={}
		prof_cont=0
		for jk in range(1,len(layers)+1):
			if jk==1:
				prof_cont=layers[jk-1]*0.5
				z_real=z0_level-prof_cont
			elif jk>1:
				prof_cont=prof_cont+layers[jk-1]*0.5+layers[jk-2]*0.5
				z_real=z0_level-prof_cont
			self.rock_dict[jk]=[self.color_dict[jk][0][0],self.color_dict[jk][1],\
			self.color_dict[jk][2],self.color_dict[jk][0][0],z_real,self.layers[jk-1]]

	def regular_mesh(self):
		"""Genera malla regular en en toda la extension de la region definida por Xmin,Xmax,Ymin y Ymax
		"""
		x_regular=range(self.Xmin+self.x_from_boarder,self.Xmax+self.x_space-self.x_from_boarder,self.x_space)
		y_regular=range(self.Ymin+self.y_from_boarder,self.Ymax+self.y_space-self.y_from_boarder,self.y_space)

		x_regular_small=range(self.x_gap_min,self.x_gap_max+self.x_gap_space,self.x_gap_space)
		y_regular_small=range(self.y_gap_min,self.y_gap_max+self.y_gap_space,self.y_gap_space)
		self.mesh_array=[]
		for nx in x_regular:
			for ny in y_regular:
				if ((nx<self.x_gap_min) or (nx>self.x_gap_max)) or ((ny<self.y_gap_min) or (ny>self.y_gap_max)):
					self.mesh_array.append([nx,ny])


		#Small polygon area must be here

		for nxx in x_regular_small:
			cnt=0
			for nyy in y_regular_small:
				if [nxx,nyy] not in self.mesh_array:
					if self.inner_mesh_type=='honeycomb':
						if cnt%2==0:
							self.mesh_array.append([nxx,nyy])
						else:
							self.mesh_array.append([nxx+self.x_gap_space/2,nyy])
					elif self.inner_mesh_type=='regular':
						self.mesh_array.append([nxx,nyy])
					cnt+=1

		if self.rotate:
			angle=self.angle
			for pair in range(len(self.mesh_array)):
				x1=self.mesh_array[pair][0]-self.Xmin
				y1=self.mesh_array[pair][1]-self.Ymin
				self.mesh_array[pair][0]=x1*math.cos(math.pi*angle/180)-y1*math.sin(math.pi*angle/180)+self.Xmin
				self.mesh_array[pair][1]=x1*math.sin(math.pi*angle/180)+y1*math.cos(math.pi*angle/180)+self.Ymin

		return np.array(self.mesh_array)

	def check_in_out(self,point,source):
		"""Verifica si un punto de la malla del campo cercano esta dentro o fuera del poligo definido por el shapefile de entrada o del campo cercano
		"""

		boolean=False
		if source=='shapefile':
			polygon=self.polygon
			cnt=0
			for n in range(len(polygon)):
				if n+1!=len(polygon):
					m,b=plb.polyfit([polygon[n][0],polygon[n+1][0]],[polygon[n][1],polygon[n+1][1]],1)
					val_range=[polygon[n][1],polygon[n+1][1]]
				elif n+1==len(polygon):
					m,b=plb.polyfit([polygon[-1][0],polygon[0][0]],[polygon[-1][1],polygon[0][1]],1)
					val_range=[polygon[-1][1],polygon[0][1]]
				
				x=(point[1]-b)/m
				if point[0]<x and min(val_range)<point[1] and point[1]<max(val_range):
					cnt+=1
			if cnt==1:
				boolean=True
		elif source=='inner':
			Xarray_inner=np.array([self.x_gap_min,self.x_gap_max+self.x_gap_space])
			Yarray_inner=np.array([self.y_gap_min,self.y_gap_max+self.y_gap_space])
			if Xarray_inner[0]<point[0] and Xarray_inner[1]>point[0] and Yarray_inner[0]<point[1] and Yarray_inner[1]>point[1]:
				boolean=True
		return boolean

	def reg_pol_mesh(self):
		"""Crea malla regular cuando existe un poligono de entrada
		"""
		x_regular=np.arange(self.Xmin+self.x_from_boarder,self.Xmax+self.x_space-self.x_from_boarder,self.x_space)
		y_regular=np.arange(self.Ymin+self.y_from_boarder,self.Ymax+self.y_space-self.y_from_boarder,self.y_space)

		self.mesh_array=[]
		for nx in x_regular:
			for ny in y_regular:
				self.mesh_array.append([nx,ny])

		if self.rotate:
			angle=self.angle
			for pair in range(len(self.mesh_array)):
				x1=self.mesh_array[pair][0]-self.Xmin
				y1=self.mesh_array[pair][1]-self.Ymin
				self.mesh_array[pair][0]=x1*math.cos(math.pi*angle/180)-y1*math.sin(math.pi*angle/180)+self.Xmin
				self.mesh_array[pair][1]=x1*math.sin(math.pi*angle/180)+y1*math.cos(math.pi*angle/180)+self.Ymin


		x_pol=[]
		y_pol=[]
		for n in range(len(self.polygon)):
			x_pol.append(int(self.polygon[n][0]))
			y_pol.append(int(self.polygon[n][1]))
		x_pol.append(int(self.polygon[0][0]))
		y_pol.append(int(self.polygon[0][1]))


		x_gap_min=min(x_pol)
		x_gap_max=max(x_pol)
	
		y_gap_min=min(y_pol)
		y_gap_max=max(y_pol)

		small_mesh=[]

		x_regular_small=np.arange(x_gap_min,x_gap_max+self.x_gap_space,self.x_gap_space)
		y_regular_small=np.arange(y_gap_min,y_gap_max+self.y_gap_space,self.y_gap_space)

		for nxx in x_regular_small:
			cnt=0
			for nyy in y_regular_small:
				if [nxx,nyy] not in small_mesh:
					if self.inner_mesh_type=='honeycomb':
						if cnt%2==0:
							small_mesh.append([nxx,nyy])
						else:
							small_mesh.append([nxx+self.x_gap_space/2,nyy])
					elif self.inner_mesh_type=='regular':
						small_mesh.append([nxx,nyy])
					cnt+=1

		if self.rotate:
			angle=self.angle
			for pair in range(len(small_mesh)):
				x1=small_mesh[pair][0]-self.x_gap_min
				y1=small_mesh[pair][1]-self.y_gap_min
				small_mesh[pair][0]=x1*math.cos(math.pi*angle/180)-y1*math.sin(math.pi*angle/180)+self.x_gap_min
				small_mesh[pair][1]=x1*math.sin(math.pi*angle/180)+y1*math.cos(math.pi*angle/180)+self.y_gap_min
				
		to_delete=[]
		for v in range(len(self.mesh_array)):
			point=[self.mesh_array[v][0],self.mesh_array[v][1]]
			check=self.check_in_out(point,source='shapefile')
			if check:
				to_delete.append(v)

		self.mesh_array=np.delete(self.mesh_array, to_delete, 0)


		to_delete=[]
		for v in range(len(small_mesh)):
			point=[small_mesh[v][0],small_mesh[v][1]]
			check=self.check_in_out(point,source='shapefile')
			if not check:
				to_delete.append(v)

		small_mesh=np.delete(small_mesh, to_delete, 0)

		mesh_pol=[]
		for vk in range(len(self.mesh_array)):
			mesh_pol.append([self.mesh_array[vk][0],self.mesh_array[vk][1]])

		for vk in range(len(small_mesh)):
			mesh_pol.append([small_mesh[vk][0],small_mesh[vk][1]])

		return np.array(mesh_pol)

	def radius_select(self,x0,y0,xr,yr):
		"""Verifica si dos puntos estan mas cerca que el criterio seleccionado
		"""
		r=((x0-xr)**2+(y0-yr)**2)**0.5
		if r<self.radius_criteria:
			boolean=1
		else:
			boolean=0
		return boolean

	def from_leapfrog_mesh(self):
		"""Extrae puntos mas extremos y la posicion de los elementos de un set de datos provinientes de leapfrog, sin embargo no considera los elementos asociados a la roca ATM 0 
		"""
		geometry_file="../mesh/from_leapfrog/LF_geometry.dat"
		leapfrog_t2_file="../mesh/from_leapfrog/LF_t2.dat"

		#Creates a dictionary using the layers
		printlayer=False
		layer_min=[]
		layers={}
		with open(geometry_file,'r') as f:
		  for line in f.readlines():

		  	if line.rstrip()=='LAYERS':
		  		printlayer=True
		  		continue
		  	elif line.rstrip()=='SURFA' or line.rstrip()=='':
		  		printlayer=False

		  	if printlayer:
		  		layer=line.rstrip()[0:2]
		  		if layer==' 0':
		  			layer_min.append(line.rstrip()[2:13])
		  			layer_middle=line.rstrip()[13:23]
		  			layer_max=line.rstrip()[13:23]
		  			continue
		  		else:
		  			layer_max=layer_min[-1]
		  			layer_min.append(line.rstrip()[2:13])
		  			layer_middle=line.rstrip()[13:23]
		  		layers[int(layer)]=[float(layer_max),float(layer_middle),float(layer_min[-1])]

		max_layer=max(layers.keys())

		xc=[]
		yc=[]
		self.LP_mesh=[]
		printeleme=False
		#Takes the elements at the selected leyer
		with open(leapfrog_t2_file,'r') as f:
		  for line in f.readlines():

		  	if line.rstrip()=='ELEME':
		  		printeleme=True
		  		continue
		  	elif line.rstrip()=='CONNE' or line.rstrip()=='':
		  		printeleme=False

		  	if printeleme and line.rstrip()[0:5]!="ATM 0" and int(line.rstrip()[3:5])==max_layer:
		  		xc=float(line.rstrip()[51:60])
		  		yc=float(line.rstrip()[60:70])
		  		self.LP_mesh.append([xc,yc])


		#Creates a dictionary of the VERTICES

		printvertices=False
		self.x_min=1E20
		self.x_max=0

		self.y_min=1E20
		self.y_max=0
		with open(geometry_file,'r') as f:
		  for line in f.readlines():

		    #It will read between the keywords GRID and CONNECTIONS
		    if line.rstrip()=='VERTICES':
		      printvertices=True
		      continue
		    elif line.rstrip()=='GRID' or line.rstrip()=="":
		      printvertices=False

		    if printvertices:
		     	vertice_x=float(line.rstrip()[4:13])
		     	vertice_y=float(line.rstrip()[13:23])

		     	if vertice_x>self.x_max:
		     		self.x_max=vertice_x

		     	if vertice_y>self.y_max:
		     		self.y_max=vertice_y

		     	if vertice_x<self.x_min:
		     		self.x_min=vertice_x

		     	if vertice_y<self.y_min:
		     		self.y_min=vertice_y

		return self.x_max,self.x_min, self.y_min,self.y_max, self.LP_mesh

	def data(self):
		"""Define los puntos que ingresaran al archivo de entrada para amesh. Adicionalmente, en caso de definir un linea en el archivo de entrada <i><lines_data/i> se procedera a ingresar estos puntos y crear puntos paralelos en ambos extremos de la linea
		"""
		self.raw_data=np.genfromtxt(self.filepath+self.filename,dtype={'names':('ID','MD','X','Y','Z','TYPE'),'formats':('<U7','f4','f4','f4','f4','<U10')},delimiter=',',skip_header=True)
		print(self.raw_data)
		self.IDXY={}

		if not self.from_leapfrog:
			if self.with_polygon:
				regular_mesh=self.reg_pol_mesh()
			else:
				regular_mesh=self.regular_mesh()
		else:
			x,x1,y1,y2,regular_mesh=self.from_leapfrog_mesh()
		
		for n in range(len(self.raw_data['ID'])):
			#Store the data from wells
			self.IDXY["%s"%(str(self.raw_data['ID'][n]))]=[self.raw_data['X'][n],self.raw_data['Y'][n],self.raw_data['TYPE'][n]]
			#self.IDXY["%s"%(str(self.raw_data['ID'][n])).split("'")[1]]=[self.raw_data['X'][n],self.raw_data['Y'][n],self.raw_data['TYPE'][n]]
			to_delete=[]
			
			x0=self.raw_data['X'][n]
			y0=self.raw_data['Y'][n]
			
			#Delete the regular points close to the wells
			for ngrid in range(len(regular_mesh)):
				if abs(x0-regular_mesh[ngrid][0])<self.radius_criteria or abs(y0-regular_mesh[ngrid][1])<self.radius_criteria:
					boolean=self.radius_select(x0,y0,regular_mesh[ngrid][0],regular_mesh[ngrid][1])
					if boolean==1:
						to_delete.append(ngrid)

			regular_mesh=np.delete(regular_mesh, to_delete, 0)

		cnt=0
		if len(self.line_file)>0:
			self.lines_data=np.loadtxt(self.filepath+self.line_file,dtype={'names':('ID','X','Y'),'formats':('S10','f4','f4')},delimiter=',',skiprows=1)
			lines_dict={}
			for v in self.lines_data:
				key=v[0].decode('UTF-8')
				if key not in list(lines_dict.keys()):
					lines_dict[key]=[[float(v[1]),float(v[2])]]
				else:
					lines_dict[key].append([float(v[1]),float(v[2])])

			d=self.fault_distance
			for n in lines_dict:
				x_lines=[]
				y_lines=[]
				for v in lines_dict[n]:
					x_lines.append(v[0])
					y_lines.append(v[1])
				x_lines_np=np.array([])
				y_lines_np=np.array([])

				for nv in range(len(x_lines)):
					if nv+1<len(x_lines):

						x_lines_np_local=np.linspace(x_lines[nv],x_lines[nv+1],num=20)
						y_lines_np_local=np.linspace(y_lines[nv],y_lines[nv+1],num=20)
						
						x_lines_np=np.append(x_lines_np,np.linspace(x_lines[nv],x_lines[nv+1],num=20))
						y_lines_np=np.append(y_lines_np,np.linspace(y_lines[nv],y_lines[nv+1],num=20))

						m,b=plb.polyfit(x_lines_np_local,y_lines_np_local,1)

						p_s=(-1/m)
						c=abs(((d**2)/(1+(p_s)**2))**0.5)

						x_u=[c+x for x in x_lines_np_local]
						x_d=[-c+x for x in x_lines_np_local]

						y_u=[]
						y_d=[]

						for v in range(len(x_u)):
							y_u.append(p_s*x_u[v]+y_lines_np_local[v]-p_s*x_lines_np_local[v])
							y_d.append(p_s*x_d[v]+y_lines_np_local[v]-p_s*x_lines_np_local[v])


						x_lines_np=np.append(x_lines_np,x_u)
						x_lines_np=np.append(x_lines_np,x_d)

						y_lines_np=np.append(y_lines_np,y_u)
						y_lines_np=np.append(y_lines_np,y_d)

				for n2 in range(len(x_lines_np)):

					to_delete=[]

					x0=x_lines_np[n2]
					y0=y_lines_np[n2]
					
					for ngrid in range(len(regular_mesh)):
						if abs(x0-regular_mesh[ngrid][0])<self.radius_criteria or abs(y0-regular_mesh[ngrid][1])<self.radius_criteria:
							boolean=self.radius_select(x0,y0,regular_mesh[ngrid][0],regular_mesh[ngrid][1])
							if boolean==1:
								to_delete.append(ngrid)

					regular_mesh=np.delete(regular_mesh, to_delete, 0)

				for n3 in range(len(x_lines_np)):
					self.IDXY["RG%s"%(n3+cnt)]=[x_lines_np[n3],y_lines_np[n3],'REGUL']
				cnt+=n3

		#Store the data from the regular mesh on IDXY
		for n in range(len(regular_mesh)):
			self.IDXY["RG%s"%(n+cnt)]=[regular_mesh[n][0],regular_mesh[n][1],'REGUL']

		return {'raw_data':self.raw_data,'IDXY':self.IDXY}

	def well_blk_assign(self):
		"""Asigna el nombre a cada bloque, priorizando los pozos, es decir, estos llevan los correlativos mas bajos. Por ultimo almacena dos archivo json para registro de los bloques asignados a los pozos
		"""
		self.well_names=[]
		self.blocks_PT={}
		self.well_blocks_names={}
		self.wells_correlative={}
		data_dict=self.data()


		for n in range(self.number_of_layer):
			cnt=110
			layer_num=0

			for x in data_dict['IDXY']:

				if cnt%100==0:
					blocknumber=cnt+10
					cnt=blocknumber
				else:
					blocknumber=cnt
					
				if cnt%1010==0:
					cnt=110
					blocknumber=cnt
					layer_num+=1

				cnt+=1
				string_ele=self.color_dict[n+1][0][(layer_num)]

				blockname=string_ele+str(blocknumber)

				self.well_names.append(blockname)

				self.well_blocks_names["%s"%(blockname)]=[x,\
				                      data_dict['IDXY'][x][0],\
				                      data_dict['IDXY'][x][1],\
				                      data_dict['IDXY'][x][2],\
				                      n+1,self.rock_dict[n+1][4],\
				                      self.rock_dict[n+1][2],self.rock_dict[n+1][5]]
				#print(data_dict['IDXY'][x][2]!='REGUL')
				#if data_dict['IDXY'][x][2]=='prod' or data_dict['IDXY'][x][2]=='rein':
				if data_dict['IDXY'][x][2]!='REGUL':
					#str(data_dict['IDXY'][x][2]).split("'")[1],\
					self.blocks_PT["%s"%(blockname)]=[x,\
				                      str(data_dict['IDXY'][x][0]),\
				                      str(data_dict['IDXY'][x][1]),\
				                      str(data_dict['IDXY'][x][2]),\
				                      str(n+1),str(self.rock_dict[n+1][4]),\
				                      self.rock_dict[n+1][2],str(self.rock_dict[n+1][5])]
					try:
						self.wells_correlative[x]=["%s"%(blockname[1:])]
					except KeyError:
						pass
		json.dump(self.blocks_PT, open("../mesh/well_dict.txt",'w'),sort_keys=True, indent=4)
		json.dump(self.wells_correlative, open("../mesh/wells_correlative.txt",'w'),sort_keys=True, indent=4)

		return {'well_names':self.well_names, 'well_blocks_names':self.well_blocks_names}

	def input_file_to_amesh(self):
		"""Genera el archivo de entrada para amesh
		"""
		welldict=self.well_blk_assign()['well_blocks_names']
		
		if not self.from_leapfrog:
			Xarray=np.array([self.Xmin,self.Xmax])
			Yarray=np.array([self.Ymin,self.Ymax])
		else:
			xmax,xmin,ymin,ymax,regular_mesh=self.from_leapfrog_mesh()
			Xarray=np.array([xmin,xmax])
			Yarray=np.array([ymin,ymax])

		boundaries=[]
		
		for i, j in itertools.product(Xarray, Yarray):
			boundaries.append([i,j])


		if self.rotate:
			angle=self.angle
			for point_n in range(len(boundaries)):
				x1=boundaries[point_n][0]-self.Xmin
				y1=boundaries[point_n][1]-self.Ymin
				boundaries[point_n][0]=x1*math.cos(math.pi*angle/180)-y1*math.sin(math.pi*angle/180)+self.Xmin
				boundaries[point_n][1]=x1*math.sin(math.pi*angle/180)+y1*math.cos(math.pi*angle/180)+self.Ymin

		boundaries=np.array(boundaries)
		hull = ConvexHull(boundaries)

		file=open(self.filename_out, "w")
		file.write("locat\n")
		for x in sorted(welldict.keys()):
			string="{:5s}{:5d}{:20.2f}{:20.2f}{:20.2f}{:20.2f}\n".format(x,welldict[x][4],\
				                                       welldict[x][1],\
				                                       welldict[x][2],\
				                                       welldict[x][5],\
				                                       welldict[x][7])
			file.write(string)
		file.write("     \n")
		file.write("bound\n")
		for n in range(len(boundaries[hull.vertices,0])):
			string_bound=" %9.3E %9.3E\n"%(boundaries[hull.vertices,0][::-1][n],boundaries[hull.vertices,1][::-1][n])
			file.write(string_bound)

		file.write("     \n")
		file.write("toler")
		file.write("     \n")
		file.write("%10s\n"%self.toler)
		file.write("     ")
		file.close()

		fileread=open(self.filename,'r')
		os.system("amesh\n")
		string_out=fileread.read()
		files2move=['in','conne','eleme','segmt']
		for fn in files2move:
			shutil.move(fn,'../mesh/from_amesh/%s'%fn)

		return string_out

	def plot_voronoi(self):
		"""En caso de ser solicitado, grafica la  malla a partir de los archivo en la carpeta ../mesh/from_amesh
		"""
		welldict=self.well_blk_assign()['well_blocks_names']
		if os.path.isfile('../mesh/from_amesh/segmt'):
			data=np.genfromtxt('../mesh/from_amesh/segmt', dtype="f8,f8,f8,f8,i8,S5,S5", names=['X1','Y1','X2','Y2','index','elem1','elem2'],delimiter=[15,15,15,15,5,5,5])
			
			fig, ax0 = plt.subplots(figsize=(10,10))

			#Set of the principal plot
			ax0.axis([self.Xmin,self.Xmax,self.Ymin,self.Ymax])
			ax0.set_xlabel('East [m]')
			ax0.set_ylabel('North [m]')
			ax0.set_title("Mesh for layer %s"%self.layer_to_plot,y=1.04)
			ax0.set_xticks(range(self.Xmin+self.x_from_boarder,self.Xmax+self.x_space-self.x_from_boarder,self.x_space))
			ax0.set_yticks(range(self.Ymin+self.y_from_boarder,self.Ymax+self.y_space-self.y_from_boarder,self.y_space))
			ax0.xaxis.set_minor_locator(AutoMinorLocator())
			ax0.yaxis.set_minor_locator(AutoMinorLocator())

			#Plot of the Yaxis in the top 
			ax1 = ax0.twinx()
			ax1.set_ylim(ax0.get_ylim())
			ax1.set_yticks(ax0.get_yticks())
			ax1.yaxis.set_minor_locator(AutoMinorLocator())

			#Plot of the Xaxis in the top 
			ax2 = ax0.twiny()
			ax2.set_xticks(ax0.get_xticks())
			ax2.set_xlim(ax0.get_xlim())
			ax2.xaxis.set_minor_locator(AutoMinorLocator())


			for n in np.unique(data['elem1']):
				Xfill=[]
				Yfill=[]
				if n[0:2] in self.color_dict[self.layer_to_plot][0]:
					for j in range(len(data['X1'])):
						cnt=0
						if data['elem1'][j]==n:
							Xfill.append(data['X1'][j])
							Xfill.append(data['X2'][j])
							Yfill.append(data['Y1'][j])
							Yfill.append(data['Y2'][j])
							plt.plot([data['X1'][j],data['X2'][j]],[data['Y1'][j],data['Y2'][j]],'-k')
							
							if self.plot_names:
								if cnt==0:
									if welldict[n][3]=='WELL':
										ax0.text(welldict[n][1],welldict[n][2],\
											      "%s, %s"%(welldict[n][0],n),fontsize=8)
										color_dot='r'
									else:
										ax0.text(welldict[n][1],welldict[n][2],\
											      welldict[n][0],fontsize=8)
										color_dot='b'
								cnt+=1

							if self.plot_centers:
								if welldict[n][3]=='WELL':
									color_dot='r'
								else:
									color_dot='b'
								ax0.plot(welldict[n][1],welldict[n][2],'%so'%color_dot,ms=1) 
								

					ax0.fill(Xfill,Yfill,welldict[n][6],alpha=0.5)
			ax0.grid()
			ax0.grid(which='major', color='#CCCCCC', linestyle='--',alpha=0.5)
			fig.savefig('../mesh/plot_voronoi_'+str(datetime.datetime.now().strftime('%Y-%m-%d_%H%M%S')),bbox_inches="tight", pad_inches = 0)
			plot_s=plt.show()
		else:
			plot_s="Excecute input_file_to_amesh() function first"
		return plot_s

	def to_steinar(self):
		"""Convierte los archivos de salida amesh en formato de entrada para Steinar
		"""
		if os.path.isfile('../mesh/from_amesh/segmt'):
			shutil.copyfile('../mesh/from_amesh/segmt', '../mesh/to_steinar/segmt')
			ele_file_st=open('../mesh/to_steinar/eleme','w')  
			ele_file=open('../mesh/from_amesh/eleme','r')  
			data_eleme=ele_file.readlines()
			
			if self.set_inac_from_poly or self.set_inac_from_inner:
				source_file="../mesh/from_amesh/eleme"
				data_eleme=pd.read_csv(source_file,delim_whitespace=True,skiprows=1,header=None,names=['block','rocktype','vol','X','Y','Z'])
				data_eleme.set_index('block')
				ele_file_st.write("eleme\n")

				if self.set_inac_from_inner:
					source='inner'
				if self.set_inac_from_poly:
					source='shapefile'
					
				for index, row in data_eleme.iterrows():
					check=self.check_in_out([row['X'],row['Y']],source=source)
					outside=1
					if not check:
						outside=-1
					ele_file_st.write("%5s%10s%5s%10.3E\n"%(row['block']," ",row['rocktype'],row['vol']*outside))
			else:
				cnt=0
				for a_line in data_eleme:
					if cnt!=0:
						name_rock_vol = a_line[0:30]  
						ele_file_st.write("%s\n"%name_rock_vol)
					else:
						ele_file_st.write("eleme\n")
					cnt+=1
			ele_file_st.close()
			ele_file.close()

			conne_file_st=open('../mesh/to_steinar/conne','w')  
			conne_file=open('../mesh/from_amesh/conne','r')  
			data_conne=conne_file.readlines()
			cnt=0
			for a_line in data_conne:
				if cnt!=0:
					name_conne = a_line[0:11]  
					if '*' in name_conne:
						pass
					elif cnt<=len(data_conne)-2:
						try:
							conne_file_st.write("%s"%name_conne)
							conne_file_st.write("   0    0    0   ")

							nod4=a_line[60:64]

							if len(nod4)==4:
								per=3
							else:
								per=a_line[29:30]

							conne_file_st.write("%s"%per)

							nod1=a_line[30:40].replace('+', '')
							conne_file_st.write(" %9.3E"%(float(nod1)))

							nod2=a_line[40:50].replace('+', '')
							conne_file_st.write(" %9.3E"%(float(nod2)))

							nod3=a_line[50:60].replace('+', '')
							conne_file_st.write(" %9.3E"%(float(nod3)))

							if len(nod4)==4:
								#nod4=-1
								conne_file_st.write("{:9.1f}".format(float(nod4)))
							elif len(nod4)==1:
								conne_file_st.write("      0.0")

							conne_file_st.write(" 0.0000E+00\n")

						except ValueError:
							pass
				else:
					conne_file_st.write("conne\n")
				cnt+=1
			conne_file_st.close()
			conne_file.close()

			in_file_st=open('../mesh/to_steinar/in','w')

			#block_file=open('block_name.csv','w')
			in_file=open('../mesh/from_amesh/in','r')  
			data_in=in_file.readlines()
			in_file_st.write("bound\n")

			if not self.from_leapfrog:
				Xarray=np.array([self.Xmin,self.Xmax])
				Yarray=np.array([self.Ymin,self.Ymax])
			else:
				xmax,xmin,ymin,ymax,regular_mesh=self.from_leapfrog_mesh()
				Xarray=np.array([xmin,xmax])
				Yarray=np.array([ymin,ymax])

			in_file_st.write(" %6.2d %6.2d\n"%(Xarray[0],Yarray[0]))
			in_file_st.write(" %6.2d %6.2d\n"%(Xarray[0],Yarray[1]))
			in_file_st.write(" %6.2d %6.2d\n"%(Xarray[1],Yarray[1]))
			in_file_st.write(" %6.2d %6.2d\n\n"%(Xarray[1],Yarray[0]))
			in_file_st.write("locat\n")

			cnt=0

			for ny in data_in:
				if cnt>0 and cnt<(len(data_in)-10):
					read1=ny[0:6]
					read2=ny[8:11]
					read3=ny[11:31]
					read4=ny[31:52]
					read5=ny[51:72]
					read6=ny[71:92]
					in_file_st.write("%s"%read1)
					in_file_st.write("%4s"%read2)
					in_file_st.write("%23.8E"%float(read3))
					in_file_st.write("%24.8E"%float(read4))
					in_file_st.write("%24.8E"%float(read5))
					in_file_st.write("%24.8E\n"%float(read6))

					#stringx="%.2f,%.2f,%.2f\n"%(float(read3),float(read4),float(read5))
					#block_file.write(stringx)
				cnt+=1

			in_file_st.close()
			in_file.close()
			#block_file.close()
			rock_file_st=open('../mesh/to_steinar/rocks','w')
			rock_file_st.write("rock1    02.6500E+031.0000E-011.0000E-151.0000E-151.0000E-152.1000E+008.5000E+02 160 243 150")
			rock_file_st.close()
		else:
			pass

	def to_GIS(self):
		"""Toma como entrada el archivo segment de ../mesh/from_amesh y eleme de ../mesh/to_stainar y los convierte en shapefile con atributos de roca, nombre y volumen
		"""
		if os.path.isfile('../mesh/from_amesh/segmt') and os.path.isfile('../mesh/to_steinar/eleme'):
			if self.plot_all_GIS:
				max_layer=self.number_of_layer
				min_layer=1
			else:
				max_layer=self.layer_to_plot
				min_layer=self.layer_to_plot

			for ln in range(min_layer,max_layer+1,1):
				w=shapefile.Writer('../mesh/GIS/mesh_cga_layer_%s'%ln)
				w.field('BLOCK_NAME', 'C', size=5)
				w.field('ROCKTYPE', 'C', size=5)
				w.field('VOLUMEN', 'F', decimal=10)
				
				alttype = np.dtype([('X1', '<f8'), ('Y1', '<f8'), ('X2', '<f8'), ('Y2', '<f8'), ('index', '<f8'), ('elem1', 'U5'), ('elem2', 'U5')])
				data = np.genfromtxt('../mesh/to_steinar/segmt', dtype=alttype, delimiter=[15,15,15,15,5,5,5])

				elem_file = open('../mesh/to_steinar/eleme', 'r')
				plain_elemfile=elem_file.read()

				for n in np.unique(data['elem1']):
					if n[0:2] in self.color_dict[ln][0]:
						points=[]
						line = re.findall(r"%s.*$"%n, plain_elemfile,re.MULTILINE)
						rocktype=str(line)[17:22]
						volumen=float(str(line)[22:32])
						for j in range(len(data['X1'])):
							if data['elem1'][j]==n:
								point1=[data['X1'][j],data['Y1'][j]]
								point2=[data['X2'][j],data['Y2'][j]]
								points.append(point1)
								points.append(point2)
						w.poly([points])
						w.record(n,rocktype,volumen)
				w.close()
				elem_file.close()
		else:
			None

def mesh_creation_func(dictionary=mesh_setup,model_conf_dictionary=input_data):
	
	"""Genera malla

	Parameters
	----------
	filename : str
		Nombre de archivo con ubicaciones de pozos (zonas de alimentacion)
	filepath : str
		Direccion donde se encuentran los archivos de entrada en general "../input"
	Xmin  : float
		Posicion X minima de malla
	Xmax : float
		Posicion X maxima de malla
	Ymin : float
		Posicion Y minima de malla
	Ymax : float
		Posicion Y maxima de malla
	toler : float
		Parametro Amesh para creacion de malla
	layers : dictionary
		Nombre y espesor de capas
	layer_to_plot : int
		En caso de graficar una capa utilizando matplotlib define la capa que se graficara
	x_space : float
		Distanciamiento horizontal de elementos en direccion X
	y_space : float
		Distanciamiento horizontal de elementos en direccion Y
	radius_criteria: float
		Distancia minima entre pozos y elementos de malla regular
	x_from_boarder: float
		Distancia horizontal donde se situara el primer elemento desde el borde
	y_from_boarder: float
		Distancia vertical donde se situara el primer elemento desde el borde
	x_gap_min: float
		Posicion X minima de malla de campo cercano
	x_gap_max: float
		Posicion X maxima de malla de campo cercano
	x_gap_space: float
		Distanciamiento horizontal de elementos en direccion X en campo cercano
	y_gap_min: float
		Posicion Y minima de malla de campo cercano
	y_gap_max: float
		Posicion Y maxima de malla de campo cercano
	y_gap_space: float
		Distanciamiento vertical de elementos en direccion X en campo cercano
	plot_names: bool
		En caso de generar grafico utiliznado matplotlib activa la representacion de los nombres de los bloques [0,1]
	plot_centers: bool
		En caso de generar grafico utiliznado matplotlib activa la representacion centros de los bloques con un punto [0,1]
	z0_level: float
		Nivel de referencia para capas
	mesh_creation: bool
		Ejecuta creacion de malla
	plot_layer: bool
		Genera grafico de capa seleccionada
	to_steinar: bool
		Genera archivos de entrada para RockEditor steinar
	to_GIS: bool
		Genera un archivo shapefile de la capa seleccionada
	plot_all_GIS: bool
		Genera un archivo shapefile para todas las capas
	from_leapfrog: bool
		lee archivos leapfrong ../mesh/from_leapfrog/LF_geometry.dat y ../mesh/from_leapfrog/LF_t2.dat, sin embargo se pierde la simbologia usada en leapfrog y no te utiliza la malla regular ni los pozos. Solamente se crea la malla utilizando amesh
	line_file: str
		En el caso de querer especificar una linea o curva por la cual se quieran generar elementos, especificar punto xy con las cabezeras: ID, X, Y en formato csv. Donde ID corresponde a un mismo elementos, por lo que para formar una linea se deber tener al menos un ID y dos pares xy
	fault_distance: float
		En caso de definir line_file, se crearan elementos paralelos a una distancia fault_distance a lo largo de los elementos
	with_polygon: bool
		De ser cierto leera un archivo shapefile dentro del cual se construira el campo cercano
	polygon_shape: str
		Archivo que delimita la geometria del campo cercano, este no debe contener concavidades internas.
	set_inac_from_poly: bool
		Establece como elementos inactivos a los que se encuentren afuera del poligono definido fuera del shapefile de entrada
	set_inac_from_inner:bool
		Establece como elementos inactivos a los que se encuentren afuera del poligono del campo cerano

	Returns
	-------
	file
	  eleme: elementos de malla especificados en formato TOUGH2, tanto en el formato AMESH como para lectura de steinar
	file
	  conne : conexiones de elementos de malla especificados en formato TOUGH2, tanto en el formato AMESH como para lectura de steinar
	shapefile
	  mesh_{field}_layer_{layer} : archivo shapefile de una o todas las capas, especialmente util cuando ya se ha definido la distribucion de roca.
	graph
	  En caso de ser especificados
	  
	Attention
	---------
	El ejecutable amesh debe estar en la carpeta scripts o en el directorio
	"""

	layers_thick=list(map(float,np.array(list(model_conf_dictionary['LAYERS'].values()))[:][:,1]))


	#layers_thick=map(float,np.array(layers.values())[:][:,1]) #Python 2

	blocks=py2amesh(dictionary['filename'],dictionary['filepath'],dictionary['Xmin'],dictionary['Xmax'],dictionary['Ymin'],dictionary['Ymax'],\
			dictionary['toler'],layers_thick,dictionary['layer_to_plot'],dictionary['x_space'],dictionary['y_space'],dictionary['radius_criteria'],dictionary['x_from_boarder'],dictionary['y_from_boarder'],\
			dictionary['x_gap_min'],dictionary['x_gap_max'],dictionary['x_gap_space'],dictionary['y_gap_min'],dictionary['y_gap_max'],dictionary['y_gap_space'],dictionary['plot_names'],dictionary['plot_centers'],\
			input_data['z_ref'],dictionary['plot_all_GIS'],dictionary['from_leapfrog'],dictionary['line_file'],dictionary['fault_distance'],dictionary['with_polygon'],dictionary['polygon_shape'],\
			dictionary['set_inac_from_poly'],dictionary['set_inac_from_inner'],dictionary['rotate'],dictionary['angle'],dictionary['inner_mesh_type'])

	if dictionary['mesh_creation']:

		blocks.input_file_to_amesh()

		if dictionary['plot_layer']:
			blocks.plot_voronoi()
		if dictionary['to_steinar']:
			blocks.to_steinar()
		if dictionary['to_GIS']:
			blocks.to_GIS()

	elif not dictionary['mesh_creation']:
		if dictionary['plot_layer']:
			blocks.plot_voronoi()
		if dictionary['to_steinar']:
			blocks.to_steinar()
		if dictionary['to_GIS']:
			blocks.to_GIS()

	return None

def change_ref_elevation(variation=0):

	in_file_steinar='../mesh/to_steinar/in'
	in_file_amesh='../mesh/from_amesh/in'

	if os.path.isfile(in_file_steinar) and os.path.isfile(in_file_amesh):

		steinar_colums=[(0,5),(5,10),(10,35),(35,60),(60,85),(85,110)]
		steinar_data=pd.read_fwf(in_file_steinar,colspecs=steinar_colums,skiprows=7,header=None,names=['block','level','X','Y','Z','h'])
		steinar_data['Z']=steinar_data['Z'] + variation

		string=""
		with open(in_file_steinar,'r') as f:
			for line in f.readlines():
				string+=line
				if 'locat' in line:
					break

		for index, row in steinar_data.iterrows():
			string+="{:5s}{:5d}{:23.8f}{:24.8f}{:24.8f}{:24.8f}\n".format(row['block'], row['level'],row['X'],row['Y'],row['Z'],row['h'])


		steinar_in_file=open(in_file_steinar,'w')
		steinar_in_file.write(string)
		steinar_in_file.close()


		amesh_colums=[(0,5),(5,10),(10,30),(30,50),(50,70),(70,90)]
		amesh_data=pd.read_fwf(in_file_amesh,colspecs=amesh_colums,skiprows=1,header=None,names=['block','level','X','Y','Z','h'])
		amesh_data.dropna(inplace=True)
		amesh_data['Z']=amesh_data['Z'] + variation

		string="locat\n"

		for index, row in amesh_data.iterrows():
			string+="{:5s}{:5.0f}{:20.3f}{:20.3f}{:20.3f}{:20.3f}\n".format(row['block'], row['level'],row['X'],row['Y'],row['Z'],row['h'])

		with open(in_file_amesh,'r') as f:
			check_in=False
			for line in f.readlines():
				if 'bound' in line:
					check_in=True
					string+='\n'
				
				if check_in:
					string+=line

		amesh_in_file=open(in_file_amesh,'w')
		amesh_in_file.write(string)
		amesh_in_file.close()

def empty_mesh():
	"""Vacia carpetas relacionadas con malla: mesh/to_steinar, mesh/from_amesh y mesh/GIS
	
	Examples
	--------
	>>> empty_mesh()

	"""
	folders=['../mesh/to_steinar','../mesh/from_amesh','../mesh/GIS']
	for folder in folders:
		for file in os.listdir(folder):
			try:
				file_path=os.path.join(folder,file)
				os.remove(file_path)
			except IsADirectoryError:
				pass

def ELEM_to_json():
	"""Combina los archivos eleme e in de la carpeta steinar en un archivo json

	Returns
	-------
	file
	  in: de la carpeta steinar
	  eleme: de la carpeta steinar
	  
	Attention
	---------
	Se utiliza la carpeta steinar ya que es la que provee la inofrmacion del modelo con las rocas modificadas.

	Examples
	--------
	>>> ELEM_to_json()
	"""
	source_file="../mesh/to_steinar/in"
	eleme_file="../mesh/to_steinar/eleme"

	eleme_dict={}
	col_eleme=[(0,6),(15,20),(20,30)]

	if os.path.isfile(source_file) and os.path.isfile(eleme_file):
		data_in=pd.read_csv(source_file,delim_whitespace=True,skiprows=7,header=None,names=['block','li','X','Y','Z','h'])
		data_eleme=pd.read_fwf(eleme_file,colspecs=col_eleme,skiprows=1,header=None,names=['block','rocktype','vol'])

		#data_eleme[['rocktype','vol']] =data_eleme.data.apply(lambda x: pd.Series([str(x)[0:5],str(x)[5:16]]))
		#data_eleme.drop('data', axis=1, inplace=True)

		data_in.set_index('block')
		data_eleme.set_index('block')

		data_eleme=data_eleme.merge(data_in, left_on='block', right_on='block',how='left')
		data_eleme.set_index('block',inplace=True)
		data_eleme.to_json('../mesh/ELEME.json',orient="index",indent=2)

	else:
		sys.exit("The file %s or directory do not exist"%source_file)

def CONNE_to_json():
	"""Genera un archivo json a partir del archivo CONNE en la carpeta to_steinar

	Returns
	-------
	file
	  conne: de la carpeta steinar
	  
	Attention
	---------
	Se utiliza la carpeta steinar ya que es la que provee la inofrmacion del modelo con las rocas modificadas.

	Examples
	--------
	>>> CONNE_to_json()
	"""
	source_file="../mesh/to_steinar/conne"

	conne_dict={}

	if os.path.isfile(source_file):
		with open(source_file) as rock_file:
			for line in rock_file.readlines()[1:]:
				linex=line.split()
				if float(linex[8])<=0:
					conne_dict[linex[0]]=[int(linex[4]),float(linex[5]),float(linex[6]),float(linex[7]),float(linex[8])]
				else:
					conne_dict[linex[0]]=[int(linex[4]),float(linex[5]),float(linex[6]),float(linex[7]),-1*float(linex[8])]

		CONNE_pd=pd.DataFrame.from_dict(conne_dict,orient='index', columns=['ISOT','D1','D2','AREAX', 'BETAX'])
		CONNE_pd.to_json('../mesh/CONNE.json',orient="index",indent=2)

	else:
		sys.exit("The file %s or directory do not exist"%source_file)

def segmnt_to_json():

	eleme_file="../mesh/to_steinar/eleme"
	segmt_file="../mesh/to_steinar/segmt"
	elment_dict={}

	col_eleme=[(0,6),(15,20),(20,30)]
	col_segment=[(0,15),(15,30),(30,45),(45,60),(60,65),(65,70),(70,75)]
	if os.path.isfile(segmt_file) and os.path.isfile(eleme_file):

		segmt_data=pd.read_fwf(segmt_file,colspecs=col_segment,header=None,names=['X1','Y1','X2','Y2','redunt','eleme1','eleme2'])

		eleme_data=pd.read_fwf(eleme_file,colspecs=col_eleme,skiprows=1,header=None,names=['block','rocktype','vol'])

		eleme_data=eleme_data.merge(segmt_data, left_on='block', right_on='eleme1',how='left')

		eleme_data['points'] = eleme_data[['X1','Y1','X2','Y2']].apply(lambda r: tuple(r), axis=1).apply(np.array)

		eleme_data=eleme_data.groupby(['block','rocktype','vol'])['points'].agg(lambda x: list(np.concatenate(x.values))).reset_index()

		eleme_data.set_index('block', inplace=True)

		eleme_data.to_json('../mesh/segmt.json',orient="index",indent=2)

	else:
		sys.exit("Either the file %s or %s do not exist"%(eleme_file,segmt_file))

def plot_voronoi(layer='D',plot_center=False,dictionary=mesh_setup,mark_elements=False,geners=geners,cross_section=None):

	segmt_json_file='../mesh/segmt.json'

	eleme_json_file='../mesh/ELEME.json'

	wells_corr_json_file='../mesh/wells_correlative.txt'


	if cross_section:
		x_points=cross_section['cross_section']['x']
		y_points=cross_section['cross_section']['y']

	if os.path.isfile(segmt_json_file):
		with open(segmt_json_file) as file:
		  	blocks=json.load(file)

		rocktypes={}
		cnt=1
		for block in blocks:
			if blocks[block]['rocktype'] not in rocktypes:
				rocktypes[blocks[block]['rocktype']]=dictionary['colors'][cnt]
				cnt+=1

		if os.path.isfile(eleme_json_file):
			with open(eleme_json_file) as file:
			  	ELEME=json.load(file)


		if mark_elements:
			legend='Legend'
			if os.path.isfile(wells_corr_json_file):
				with open(wells_corr_json_file) as file:
				  	well_corr=json.load(file)
			for gen in geners:
				src=geners[gen]['SL']+str(geners[gen]['NS'])
				corr=gen[1:6]
				well_corr[src]=[corr]
			corr_well = {y[0]:x for x,y in well_corr.items()}
		else:
			legend='ROCKS'

		fig, ax0 = plt.subplots(figsize=(10,10))
		ax0.set_xlabel("East [m]")
		ax0.set_ylabel("North [m]")
		
		if cross_section:
			ax0.plot(x_points,y_points,linestyle='--',color='darkred')
			ax0.text(x_points[0], y_points[0], 'A',color='darkred',horizontalalignment='left')
			ax0.text(x_points[-1], y_points[-1], "A'",color='darkred',horizontalalignment='left')


		#ax0.axis([mesh_setup['Xmin'],mesh_setup['Xmax'],mesh_setup['Ymin'],mesh_setup['Ymax']])
		colors_on_plot=[]
		legends=[]
		for block in blocks:
			if block[0]==layer:
				for i, point in enumerate(blocks[block]['points']):
					if i==0:
						fill_x=[]
						fill_y=[]
						line=[]
					if i!=0 and (i)%4==0:
						plt.plot(line[0],line[1],line[2],line[3],'-k')
						fill_x.extend([line[0],line[2]])
						fill_y.extend([line[1],line[3]])
						line=[]
					if i%4 in [0,1,2,3]:
						line.append(point)

				if plot_center:
					ax0.plot(ELEME[block]['X'],ELEME[block]['Y'],'.k')

				ax0.fill(fill_x,fill_y,facecolor=rocktypes[blocks[block]['rocktype']], edgecolor='black',alpha=0.5)
				
				if mark_elements:
					if 'red' not in  colors_on_plot:
						legend_for_rock = mpatches.Patch(color='red', label='sinks/sources',alpha=0.75)
						colors_on_plot.append('red')
						legends.append(legend_for_rock)
					if any(corr[0] in block for corr in well_corr.values()):
						ax0.fill(fill_x,fill_y,facecolor='red', edgecolor='black',alpha=0.75)
						ax0.annotate(corr_well[block[1:6]],
									xy=(ELEME[block]['X'],ELEME[block]['Y']), xycoords='data',
									xytext=(ELEME[block]['X']+500,ELEME[block]['Y']+500),textcoords='data',fontsize=10,
									arrowprops=dict(arrowstyle="->", color="white",
									shrinkA=5, shrinkB=5,
									patchA=None, patchB=None,
									connectionstyle="bar,angle=180,fraction=-0.2"))

				if rocktypes[blocks[block]['rocktype']] not in colors_on_plot:
					legend_for_rock = mpatches.Patch(color=rocktypes[blocks[block]['rocktype']], label=blocks[block]['rocktype'],alpha=0.5)
					colors_on_plot.append(rocktypes[blocks[block]['rocktype']])
					legends.append(legend_for_rock)

		plt.subplots_adjust(right=0.8)
		plt.legend(handles=legends,title=legend, bbox_to_anchor=(1.01, 1), loc='upper left')
		plt.axis('equal')
		plt.savefig('layer_C.png',dpi=300,transparent=False,bbox_inches='tight')
		plt.show()
	else:
		sys.exit("The file %s does not exist"%(segmt_json_file))

def plot_3D_mesh():

	segmt_file='../mesh/from_amesh/segmt'

	if os.path.isfile(segmt_file):
		with open('../mesh/ELEME.json') as file:
	  		blocks_position=json.load(file)

		elements_count = {blocks:{'count':[],'points':[]} for blocks in blocks_position.keys()}

		layer_dictionary={}
		layers_info=geometry.vertical_layers()
		for i, name in enumerate(layers_info['name']):
			layer_dictionary[name]=[layers_info['top'][i],layers_info['bottom'][i]]


		with open(segmt_file,'r') as segmt_file:
			for i, line in enumerate(segmt_file):
				line_array=line.split()
				len_array=len(line_array)
				if len_array==6:
					elem2=line_array[5][5:12]
				elif len_array==7:
					elem2=line_array[5][5:12]+line_array[6]

				X1=float(line_array[0])
				Y1=float(line_array[1])
				
				X2=float(line_array[2])
				Y2=float(line_array[3])
				direction=line_array[4]
				elem1=line_array[5][0:5]
				Z=layer_dictionary[elem1[0]][0]

				if direction==1:
					elements_count[elem1]['points'].append([X1,Y1,Z])
					elements_count[elem1]['points'].append([X2,Y2,Z])
					elements_count[elem1]['count'].append(i*2)
					elements_count[elem2]['count'].append(i*2+1)
				else:
					elements_count[elem1]['points'].append([X2,Y2,Z])
					elements_count[elem1]['points'].append([X1,Y1,Z])
					if '*' in elem2:
						elements_count[elem1]['count'].append(i*2+1)
						elements_count[elem1]['count'].append(i*2)
					else:
						elements_count[elem2]['count'].append(i*2)
						elements_count[elem1]['count'].append(i*2+1)

		points=[]
		cells=[[] for i in range(len(elements_count))]

		cnt=0
		for n, elem in enumerate(elements_count):
			temp_points=list(set(map(tuple, elements_count[elem]['points'])))
			for point in temp_points:
				points.append(list(point))

				cells[n].append(cnt)
				cnt+=1

				Z2=layer_dictionary[elem[0]][1]
				point2=[point[0],point[1],Z2]
				points.append(point2)

				cells[n].append(cnt)
				cnt+=1

		celltypes = np.empty(len(elements_count), dtype=np.uint8)
		celltypes[:]=vtk.VTK_HEXAHEDRON

		"""
		for i, eleme in enumerate(elements_count):
			celltypes[i]=len(cells[i])*3/2
		"""

		cells_x = []
		for i, cell in enumerate(cells):
			cells_x.extend(cells[i])

		cells_x=np.array(cells_x)
		print(cells_x.ravel())

		grid = pv.UnstructuredGrid(cells_x, celltypes, np.array(points))
		_ = grid.plot(show_edges=True)

#mesh_creation_func()
#ELEM_to_json()
#plot_3D_mesh()
#segmnt_to_json()
#plot_voronoi()

#change_ref_elevation(variation=-600)


#import t2resfun as t2r
from model_conf import *
import sqlite3
import pandas as pd
from datetime import datetime, timedelta
import os
from model_conf import input_data
import t2_writer as t2w
from formats import formats_t2
import geometry as geomtr
import matplotlib.pyplot as plt

def write_t2_format_gener(var_array,time_array,var_type,var_enthalpy,type_flow,input_dictionary):
	"""Genera bloque de tiempos, flujos y entalpia para cada pozo

	Parameters
	----------
	var_array : array
	  Arreglo que contiene los flujos de pozo
	time_array : array
	  Arreglo que contiene una lista de fechas en formato datetime
	ref_date   :date
	  Fecha de referencia, utilizada para calcular delta de tiempo
	var_type	:str
	  Indica el tipo de pozo 'P' o 'R'
	var_enthalpy :array
	  Arreglo que contiene las entalpias para el caso de los pozos reinyectores
	type_flow :str
	  Tipo de flujo a proyectar: invariable, constant o shutdown

	Returns
	-------
	str
	  string_P: Cadena de texto que contiene la entrada TOUGH2 para un pozo productor
	str
	  string_R : Cadena de texto que contiene la entrada TOUGH2 para un pozo reinyector
	int
	  cnt_P : Numero de elementos (tiempos o flujos) contenidos en la cada string_P de pozo productor
	int
	  cnt_R : Numero de elementos (tiempos, flujos o entalpias) contenidos en la cada string_P de pozo productor
	  
	Other Parameters
	----------------
	float
		extra_time : parametro interno, tiempo en segundos, utilizado cuando el tipo de flujo es 'shutdown'

	Attention
	---------
	La longitud de var_array, time_array y var_enthalpy deben ser iguales

	Note
	----
	Para todos los casos se incorpora un flujo cero para un tiempo -1E50 y en tiempo minimo 
	Para el tipo flujo 'constant': reproduce el ultimo flujo  en un tiempo 1E50
	Para el tipo flujo 'invariable': traduce al formato TOUGH2 los arreglos sin hacer cambios
	Para el tipo flujo 'shutdown': reproduce el ultimo flujo  por un tiempo extra_time y luego igual a cero el flujo

	Examples
	--------
	>>> write_t2_format_gener(data['flow'],dates,ref_date,'P',data['flowing_enthalpy'],'constant')
	"""
	ref_date=input_dictionary['ref_date']
	string_time_R=''
	string_flow_R=''
	string_enthalpy_R=''
	string_time_P=''
	string_flow_P=''

	t_min=-1E50
	t_max=1E50
	flow_min=0
	enthalpy_min=500E3
	time_zero=0
	extra_time=365.25*24*3600/2

	if type_flow=="invariable":
		cnt_P=1
		cnt_R=1
	elif type_flow=="constant" or type_flow=="shutdown":
		string_time_P+='%14.6E'%t_min
		string_flow_P+='%14.6E'%flow_min
		string_time_R+='%14.6E'%t_min
		string_flow_R+='%14.6E'%flow_min
		string_enthalpy_R+='%14.6E'%enthalpy_min

		string_time_P+='%14.6E'%time_zero
		string_flow_P+='%14.6E'%flow_min
		string_time_R+='%14.6E'%time_zero
		string_flow_R+='%14.6E'%flow_min
		string_enthalpy_R+='%14.6E'%enthalpy_min

		cnt_P=2
		cnt_R=2


	if len(var_array)==len(time_array):
		for n in range(len(var_type)):
			timex=(time_array[n]-ref_date).total_seconds()
			if var_type[n]=='P':
				well_type="P"
				string_time_P+='%14.6E'%timex
				string_flow_P+='%14.6E'%-var_array[n]
				last_flow=-var_array[n]
				cnt_P+=1
				if cnt_P%4==0 and n!=(len(var_type)-1):
					string_time_P+='\n'
					string_flow_P+='\n'
				

			if var_type[n]=='R':
				well_type="R"
				string_time_R+='%14.6E'%timex
				string_flow_R+='%14.6E'%var_array[n]
				string_enthalpy_R+='%14.6E'%var_enthalpy[n]
				last_flow=var_array[n]
				last_enthalpy=var_enthalpy[n]
				cnt_R+=1
				if cnt_R%4==0 and n!=(len(var_type)-1):
					string_time_R+='\n'
					string_flow_R+='\n'
					string_enthalpy_R+='\n'
			last_time=timex

		if type_flow=="invariable":
			pass
		elif type_flow=="constant":
			if cnt_R>3:
				if cnt_R%4==0:
					string_time_R+='\n'
					string_flow_R+='\n'
					string_enthalpy_R+='\n'
				cnt_R+=1
				string_time_R+='%14.6E'%t_max
				string_flow_R+='%14.6E'%last_flow
				string_enthalpy_R+='%14.6E'%last_enthalpy

			if cnt_P>3:
				if cnt_P%4==0:
					string_time_P+='\n'
					string_flow_P+='\n'
				string_time_P+='%14.6E'%t_max
				string_flow_P+='%14.6E'%last_flow
				cnt_P+=1

		elif type_flow=="shutdown":
			if cnt_R>3:
				if cnt_R%4==0:
					string_time_R+='\n'
					string_flow_R+='\n'
					string_enthalpy_R+='\n'

				cnt_R+=2

				string_time_R+='%14.6E'%(last_time+extra_time)
				string_flow_R+='%14.6E'%flow_min
				string_enthalpy_R+='%14.6E'%last_enthalpy

				string_time_R+='%14.6E'%t_max
				string_flow_R+='%14.6E'%flow_min
				string_enthalpy_R+='%14.6E'%last_enthalpy

			if cnt_P>3:
				if cnt_P%4==0:
					string_time_P+='\n'
					string_flow_P+='\n'
				
				cnt_P+=2

				string_time_P+='%14.6E'%(last_time+extra_time)
				string_flow_P+='%14.6E'%flow_min

				string_time_P+='%14.6E'%t_max
				string_flow_P+='%14.6E'%flow_min

		if cnt_R==2 or cnt_R==3:
			string_time_R=''
			string_flow_R=''
			string_enthalpy_R=''

		if cnt_P==2 or cnt_P==3:
			string_time_P=''
			string_flow_P=''

		string_time_P+='\n'
		string_flow_P+='\n'
		string_time_R+='\n'
		string_flow_R+='\n'
		string_enthalpy_R+='\n'

		string_P=string_time_P+string_flow_P
		string_R=string_time_R+string_flow_R+string_enthalpy_R
		return string_P, string_R, cnt_P, cnt_R
	else:
		print("Time and variable array must have the same length")

def write_gener_from_sqlite(type_flow,forecast,wells,input_dictionary=input_data,make_up=False):
	"""Genera el bloque GENER para todos los pozos

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf
	wells : array
	  Arreglo con nombre de pozos a ser incluidos en la etapa de produccion
	type_flow :str
	  Tipo de flujo a proyectar: invariable, constant o shutdown
	forecast : bool
	   Se introducen los flujo de pozos nuevos al GENER


	Returns
	-------
	file
	  GENER_PROD: archivo de texto en direccion '../model/t2/sources/GENER_PROD' contiene toda la informacion GENER de cada  pozo
	  
	Other Parameters
	----------------
	int
		source_corr_num : los correlativos de fuentes seran generados de la siguiente forma 'SRC'+source_corr_num

	Attention
	---------
		De existir otras fuentes definidas en el archivo model_conf con el prefijo SRC deben ingresarse primero  a la base de datos 
		sqlite ejecutando la funcion write_geners_to_txt_and_sqlite, de ser asi el correlativo de fuentes sera enumerado con el valor mas alto
		de SRCXX


	Examples
	--------
	>>> write_gener_from_sqlite(db_path,wells,type_flow,'constant')
	"""

	db_path=input_dictionary['db_path']

	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	gener=''
	source_corr_num=10


	for name in sorted(wells):
		data=pd.read_sql_query("SELECT type,date_time,steam_flow+liquid_flow as m_total,flowing_enthalpy,well_head_pressure\
		 FROM mh WHERE well='%s';"%name,conn)

		dates_func=lambda datesX: datetime.strptime(datesX, "%Y-%m-%d %H:%M:%S")
		#Read file cooling
		dates=list(map(dates_func,data['date_time']))

		q_wf="SELECT MeasuredDepth,porcentage from wellfeedzone WHERE well='%s'"%name
		c.execute(q_wf)
		rows=c.fetchall()
		
		wellfeedzone=[]
		porcentage=[]
		for row in rows:
			wellfeedzone.append(row[0])
			porcentage.append(row[1])

		x,y,masl_depth=geomtr.MD_to_TVD(name,wellfeedzone)

		for feedn in range(len(porcentage)):

			#Create t the block name for each feedzone
			q_layer="SELECT correlative FROM layers where top>=%s and bottom<%s"%(masl_depth[feedn],masl_depth[feedn])

			c.execute(q_layer)
			rows=c.fetchall()
			for row in rows:
				layer_corr=row[0]

			q_corr="SELECT blockcorr FROM t2wellblock WHERE well='%s'"%(name)

			c.execute(q_corr)
			rows=c.fetchall()
			for row in rows:
				blockcorr=row[0]

			source_block=layer_corr+blockcorr #Block name for feedzone

			#Generates the source nickname

			#Extracts the maximum value of the source names
			q_corr="SELECT max(substr(source_nickname,4,2)) FROM t2wellsource"

			c.execute(q_corr)
			rows=c.fetchall()
			for row in rows:
				source_last=row[0]

			#If there is not sources name it creates one base on the increment
			if source_last==None:
				source_corr="SRC%s"%source_corr_num
			else:

				#If there  is something,  it looks for that block source nickname
				q_corr_sqlite="SELECT source_nickname FROM t2wellsource where blockcorr='%s'"%source_block

				c.execute(q_corr_sqlite)
				conn.commit()
				rows=c.fetchall()
				for row in rows:
					source_corr_num_sqlite=row[0]

				#if the block is already on the database, it creates a new name
				if len(rows)==0:
					source_corr_num=int(source_last)+1
					source_corr="SRC%s"%source_corr_num
				else:
					#If the block name exists it takes that value
					source_corr=source_corr_num_sqlite

			P,R, LTAB_P, LTAB_R= write_t2_format_gener(data['m_total'].values*porcentage[feedn],dates,data['type'],data['flowing_enthalpy']*1E3,type_flow,input_dictionary=input_dictionary)

			if type_flow=="invariable":
				condition=1
			elif type_flow=="constant":
				condition=2
			elif type_flow=="shutdown":
				condition=4

			r_check=False
			
			if LTAB_R>condition:
				if forecast :
					gener+="%s%s                %4d     MASSi\n"%(source_block,source_corr,LTAB_R)
					gener+=R
					try:
						q="INSERT INTO t2wellsource(well,blockcorr,source_nickname) VALUES ('%s','%s','%s')"%(name,source_block,source_corr)
						print(q)
						c.execute(q)
						conn.commit()
						r_check=True
					except sqlite3.IntegrityError:
						print("""Two reasons:
								 1-There is already a SOURCE nickname assign to this block (%s),
								 	in this case is better to erase the table content on table t2wellsource, 
								 	be sure about the data on wellfeedzone table
								 	and rerun this function
								 2-Skip the message if you are just updating the flowrates"""%source_block)

				else:
					gener+="%s%s                %4d     MASSi\n"%(source_block,source_corr,LTAB_R)
					gener+=R
					try:
						q="INSERT INTO t2wellsource(well,blockcorr,source_nickname) VALUES ('%s','%s','%s')"%(name,source_block,source_corr)
						c.execute(q)
						conn.commit()
						r_check=True
					except sqlite3.IntegrityError:
						print("""Two reasons:
								 1-There is already a SOURCE nickname assign to this block (%s),
								 	in this case is better to erase the table content on table t2wellsource, 
								 	be sure about the data on wellfeedzone table
								 	and rerun this function
								 2-Skip the message if you are just updating the flowrates"""%source_block)

			p_check=False
			if LTAB_P>condition:
				if r_check:
					source_corr_num+=1
					source_corr="SRC%s"%source_corr_num

				if  forecast :
					gener+="%s%s                %4d     MASS\n"%(source_block,source_corr,LTAB_P)
					gener+=P
					try:
						q="INSERT INTO t2wellsource(well,blockcorr,source_nickname) VALUES ('%s','%s','%s')"%(name,source_block,source_corr)
						c.execute(q)
						conn.commit()
						p_check=True
					except sqlite3.IntegrityError:
						print("""Two reasons:
								 1-There is already a SOURCE nickname assign to this block (%s),
								 	in this case is better to erase the table content on table t2wellsource, 
								 	be sure about the data on wellfeedzone table
								 	and rerun this function
								 2-Skip the message if you are just updating the flowrates"""%source_block)

				else:
					gener+="%s%s                %4d     MASS\n"%(source_block,source_corr,LTAB_P)
					gener+=P
					try:
						q="INSERT INTO t2wellsource(well,blockcorr,source_nickname) VALUES ('%s','%s','%s')"%(name,source_block,source_corr)
						c.execute(q)
						conn.commit()
						p_check=True
					except sqlite3.IntegrityError:
						print("""Two reasons:
								 1-There is already a SOURCE nickname assign to this block (%s),
								 	in this case is better to erase the table content on table t2wellsource, 
								 	be sure about the data on wellfeedzone table
								 	and rerun this function
								 2-Skip the message if you are just updating the flowrates"""%source_block)

			#Insert feedzones for new wells and wells with no flow during history, in case it is need it
			if not (r_check or p_check):
				try:
					q="INSERT INTO t2wellsource(well,blockcorr,source_nickname) VALUES ('%s','%s','%s')"%(name,source_block,source_corr)
					c.execute(q)
					conn.commit()
				except sqlite3.IntegrityError:
					print("""Two reasons:
							 1-There is already a SOURCE nickname assign to this block (%s),
							 	in this case is better to erase the table content on table t2wellsource, 
							 	be sure about the data on wellfeedzone table
							 	and rerun this function
							 2-Skip the message if you are just updating the flowrates"""%source_block)
	conn.close()
	if make_up:
		gener_file=open('../model/t2/sources/GENER_MAKEUP','w')
	else:
		gener_file=open('../model/t2/sources/GENER_PROD','w')
	gener_file.write(gener)
	gener_file.close()

def def_gener_selector(section,key,geners):
	try:
		value=geners[section][key]
		def_value=False
	except KeyError:
		value=formats_t2['GENER'][key][0]
		def_value=True
	return value,def_value

def write_geners_to_txt_and_sqlite(db_path=input_data['db_path'],geners=geners):
	"""Genera GENER para cada source

	Parameters
	----------
	db_path : str
	   Direccion de base de datos sqlite, tomado de model_conf
	geners : dicionary
	  Diccionario definido en model_conf con todos los parametros para un sumidero o fuente

	Returns
	-------
	file
	  GENER_SOURCES: archivo de texto en direccion '../model/t2/sources/GENER_SOURCES' contiene la informacion de cada elemento definido como fuente o sumidero en model_conf
	  
	Attention
	---------
		El archivo GENER_SOURCES es reemplazado cuando se ejecuta esta funcion y las fuentes o sumideros actualizados en la base de datos.
		Elimina los elementos que ya no estan definidos en model_conf

	Examples
	--------
	>>> write_geners_to_txt_and_sqlite(db_path,geners)
	"""

	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	list_geners=[]

	string=""
	for gener in geners:
		string+=t2w.converter(gener,formats_t2['GENER']['BLOCK'][1],def_value=False)
		list_geners.append(geners[gener]['SL']+str(geners[gener]['NS']))
		for index, key in enumerate(formats_t2['GENER']):
			if index!=0:
				value,def_value=def_gener_selector(gener,key,geners)
				string+=t2w.converter(value,formats_t2['GENER'][key][1],def_value)
		try:
			q="INSERT INTO t2wellsource(well,blockcorr,source_nickname) VALUES ('%s','%s','%s')"%(geners[gener]['TYPE'],gener,geners[gener]['SL']+str(geners[gener]['NS']))
			c.execute(q)
			conn.commit()
		except sqlite3.IntegrityError:
			print("The blockcorr %s is already used by another source"%(gener))
		string+="\n"
	#string+="\n"

	file=open('../model/t2/sources/GENER_SOURCES','w')
	file.write(string)
	file.close()

	#As the sources defined on model_conf are the only allowed any other source not incluaded on the geners dictionary will be erased
	list_geners_string=''.join(["'%s',"%str(i) for i in list_geners])
	query="DELETE FROM t2wellsource WHERE source_nickname NOT IN (%s) AND  source_nickname LIKE'GEN*' "%(list_geners_string[0:-1])
	c.execute(query)
	conn.commit()

	conn.close()

def create_well_flow(flow_times,include_gener=True,input_dictionary=input_data):
  """Crea un archivo mh para cada los pozos en flow_times, los cuales a su vez deben haber sido definidos en alguna de las secciones 'WELLS','MAKE_UP_WELLS' o 'NOT_PRODUCING_WELL' de input_data
  
  Parameters
  ----------
  flow_times : dicionary
    Diccionario contiene: nombre de pozo, tiempos (En meses), flujos vapor, flujo agua, entalpia,  presion y tipo de pozo.
  include_gener: bool
  	True convierte en formato T2 a gener y lo adiciona al final del archivo GENER_PROD
  
  Returns
  -------
  str
    string: imprime en pantalla el bloque de texto
  file
 	En la ubicacion ../input/mh/
    
  Attention
  ---------
  El dictionario debe ser ingresado manualmente y el 
  
  Examples
  --------
  Ejemplo de diccionario de entrada:
  >>> flow_times={'CHI-3A':[[0,300],
  >>> [10,10],
  >>> [45,45],
  >>> [1100,1100],
  >>> [6.9,6.9],
  >>> 'P']}
  
  """

  for name in flow_times:
    if os.path.isfile('../input/mh/%s_mh.dat'%name):
    	print("Well already exists, data will be overwrite %s"%name)
    else:
    	print("New well added %s"%name)

    file=open('../input/mh/%s_mh.dat'%name,'w')
    file.write("type,date-time,steam,liquid,enthalpy,WHPabs\n")
    for n in range(len(flow_times[name][0])):
      date=(flow_times[name][0][n]).strftime("%Y-%m-%d %H:%M:%S")
      steam=flow_times[name][1][n]
      brine=flow_times[name][2][n]
      enthlapy=flow_times[name][3][n]
      pressure=flow_times[name][4][n]
      type_well=flow_times[name][5]
      string="%s,%s,%s,%s,%s,%s\n"%(type_well,date,steam,brine,enthlapy,pressure)
      file.write(string)
  if include_gener:
  	write_gener_from_sqlite(type_flow='constant',forecast=True,wells=flow_times.keys(),make_up=True,input_dictionary=input_dictionary)

def plot_makeup_wells(flow_times):

	# Plotting
	fig = plt.figure(figsize=(8, 2))
	ax = fig.add_subplot(111)

	cnt=1
	y_positions=[]
	well_tick=[]
	for well in flow_times:
		if well[0]=='P':
			color='r'
		elif well[0]=='I':
			color='b'
		data_range=flow_times[well][0]
		value=[cnt,cnt]
		y_positions.append(cnt)
		cnt+=1
		plt.plot(data_range, value,color,lw=4,alpha=0.5)
		well_tick.append(str(well).replace("X","").replace("Z",""))

	ax.set_xlabel("Time")
	xlims=[datetime(2025,1,1),datetime(2031,1,1)]
	ax.set_xlim(xlims)
	plt.yticks(y_positions, well_tick)
	plt.show()
	fig.savefig('try.png')

#Sostenibilidad Ahuachapan
#write_gener_from_sqlite(db_path,wells,type_flow='constant',forecast=False)
#write_geners_to_txt_and_sqlite(db_path,geners)

#Escenario 1
flow_times_1={'ZAH-38A':[[datetime(2028,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [7.5,7.5],
			           [50,50],
			           [1100,1100],
			           [7,7],
			           'P'],
				'ZAH-38B':[[datetime(2028,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
				           [7.5,7.5],
				           [50,50],
				           [1100,1100],
				           [7,7],
				           'P'],
				'ZAH-38C':[[datetime(2028,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
				           [7.5,7.5],
				           [50,50],
				           [1100,1100],
				           [7,7],
				           'P'],
				'ZAH-39A':[[datetime(2029,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
				           [7.5,7.5],
				           [60,60],
				           [1100,1100],
				           [7,7],
				           'P'],
				'ZAH-39B':[[datetime(2029,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
				           [7.5,7.5],
				           [60,60],
				           [1100,1100],
				           [7,7],
				           'P'],
				'XAH-2R':[[datetime(2022,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
				           [0,0],
				           [40,40],
				           [250,250],
				           [2,2],
				           'R'],
				'XCH-9C':[[datetime(2022,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
				           [0,0],
				           [102.3,102.3],
				           [410,410],
				           [2,2],
				           'R'],
				'XCH-12A':[[datetime(2029,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
				           [0,0],
				           [60,60],
				           [675,675],
				           [2,2],
				           'R'],
				'XCH-12B':[[datetime(2029,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
				           [0,0],
				           [60,60],
				           [675,675],
				           [2,2],
				           'R'],
				'XCH-8A':[[datetime(2028,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
				           [0,0],
				           [60,60],
				           [675,675],
				           [2,2],
				           'R'],
				'XCH-8B':[[datetime(2028,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
				           [0,0],
				           [60,60],
				           [675,675],
				           [2,2],
				           'R'],
			}



#Escenario_2
flow_times_2={'ZAH-37A':[[datetime(2024,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [7.5,7.5],
			           [60,60],
			           [1100,1100],
			           [7,7],
			           'P'],
			'ZAH-37B':[[datetime(2027,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [7.5,7.5],
			           [60,60],
			           [1100,1100],
			           [7,7],
			           'P'],
			'ZAH-38A':[[datetime(2028,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [7.5,7.5],
			           [50,50],
			           [1100,1100],
			           [7,7],
			           'P'],
			'ZAH-38B':[[datetime(2028,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [7.5,7.5],
			           [50,50],
			           [1100,1100],
			           [7,7],
			           'P'],
			'ZAH-38C':[[datetime(2028,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [7.5,7.5],
			           [50,50],
			           [1100,1100],
			           [7,7],
			           'P'],
			'ZAH-39A':[[datetime(2029,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [7.5,7.5],
			           [60,60],
			           [1100,1100],
			           [7,7],
			           'P'],
			'ZAH-39B':[[datetime(2029,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [7.5,7.5],
			           [60,60],
			           [1100,1100],
			           [7,7],
			           'P'],
			'ZAH-39C':[[datetime(2029,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [7.5,7.5],
			           [60,60],
			           [1100,1100],
			           [7,7],
			           'P'],
			'XAH-2R':[[datetime(2022,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [0,0],
			           [40,40],
			           [250,250],
			           [2,2],
			           'R'],
			'XCH-9C':[[datetime(2022,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [0,0],
			           [87.225,87.225],
			           [410,410],
			           [2,2],
			           'R'],
			'CH-D':[[datetime(2022,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [0,0],
			           [60,60],
			           [675,675],
			           [2,2],
			           'R'],
			'XCH-D2':[[datetime(2025,1,1,0,0,0),datetime(2080,6,1,0,0,0)],
			           [0,0],
			           [60,60],
			           [675,675],
			           [2,2],
			           'R'],
			'XCH-12A':[[datetime(2029,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [0,0],
			           [60,60],
			           [675,675],
			           [2,2],
			           'R'],
			'XCH-12B':[[datetime(2029,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [0,0],
			           [60,60],
			           [675,675],
			           [2,2],
			           'R'],
			'XCH-8A':[[datetime(2028,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [0,0],
			           [60,60],
			           [675,675],
			           [2,2],
			           'R'],
			'XCH-8B':[[datetime(2028,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [0,0],
			           [60,60],
			           [675,675],
			           [2,2],
			           'R'],
			}

#Escenario_3
flow_times_3={'ZAH-37A':[[datetime(2024,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [7.5,7.5],
			           [60,60],
			           [1100,1100],
			           [7,7],
			           'P'],
			'ZAH-37B':[[datetime(2027,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [7.5,7.5],
			           [60,60],
			           [1100,1100],
			           [7,7],
			           'P'],
			'ZAH-38A':[[datetime(2028,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [7.5,7.5],
			           [50,50],
			           [1100,1100],
			           [7,7],
			           'P'],
			'ZAH-38B':[[datetime(2028,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [7.5,7.5],
			           [50,50],
			           [1100,1100],
			           [7,7],
			           'P'],
			'ZAH-38C':[[datetime(2028,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [7.5,7.5],
			           [50,50],
			           [1100,1100],
			           [7,7],
			           'P'],
			'ZAH-39A':[[datetime(2029,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [7.5,7.5],
			           [60,60],
			           [1100,1100],
			           [7,7],
			           'P'],
			'ZAH-39B':[[datetime(2029,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [7.5,7.5],
			           [60,60],
			           [1100,1100],
			           [7,7],
			           'P'],
			'ZAH-39C':[[datetime(2029,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [7.5,7.5],
			           [60,60],
			           [1100,1100],
			           [7,7],
			           'P'],
			'XAH-2R':[[datetime(2022,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [0,0],
			           [40,40],
			           [250,250],
			           [2,2],
			           'R'],
			'XCH-9C':[[datetime(2022,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [0,0],
			           [102.225,102.225],
			           [410,410],
			           [2,2],
			           'R'],
			'CH-D':[[datetime(2024,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [0,0],
			           [60,60],
			           [675,675],
			           [2,2],
			           'R'],
			'XCH-D1':[[datetime(2027,1,1,0,0,0),datetime(2080,6,1,0,0,0)],
			           [0,0],
			           [60,60],
			           [675,675],
			           [2,2],
			           'R'],
			'XCH-D2':[[datetime(2027,1,1,0,0,0),datetime(2080,6,1,0,0,0)],
			           [0,0],
			           [60,60],
			           [675,675],
			           [2,2],
			           'R'],
			'XCH-12A':[[datetime(2029,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [0,0],
			           [60,60],
			           [675,675],
			           [2,2],
			           'R'],
			'XCH-12B':[[datetime(2029,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [0,0],
			           [60,60],
			           [675,675],
			           [2,2],
			           'R'],
			'XCH-8A':[[datetime(2027,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [0,0],
			           [60,60],
			           [675,675],
			           [2,2],
			           'R'],
			'XCH-8B':[[datetime(2028,1,1,0,0,0),datetime(2029,1,1,0,0,0),datetime(2080,1,1,0,0,0)],
			           [0,0,0],
			           [30,60,60],
			           [675,675,675],
			           [2,2,2],
			           'R'],
			}

#Expansion

#create_constant_scenario(db_path,flow_times_3)

#write_gener_from_sqlite(db_path,wells,type_flow='constant',forecast=True)

#write_geners_to_txt_and_sqlite(db_path=input_data['db_path'],geners=geners)

#write_gener_from_sqlite(type_flow='constant',forecast=True,db_path=input_data['db_path'],wells=input_data['WELLS'])

#create_well_flow(flow_times_1,include_gener=True)

#write_geners_to_txt_and_sqlite()


from formats import formats_t2
from model_conf import input_data
import numpy as np
from datetime import datetime
import sys
import os
import pandas as pd
import sqlite3

def converter(value,format_t2,def_value):
	if 's' in format_t2:
		if value==None:
			eq_str_len=int(format_t2.replace(">","").replace("s",""))
			value=" "*eq_str_len
		else:
			value=str(value)
	elif 'E' in format_t2 and def_value:
		if value==None:
			eq_str_len=int(format_t2.split('.')[0].split('>')[1])
			value=" "*eq_str_len
			format_t2=">%ss"%eq_str_len
	elif 'd' in format_t2 and def_value:
		if value==None:
			eq_str_len=int(format_t2.split('d')[0].split('>')[1])
			value=" "*eq_str_len
			format_t2=">%ss"%eq_str_len
	output=format(value,format_t2)

	return output

def def_value_selector(section,key,input_dictionary):
	try:
		value=input_dictionary[section][key]
		def_value=False
	except KeyError:
		value=formats_t2[section][key][0]
		def_value=True
	return value,def_value

def FCG_file_checker(name):
	string=""
	if os.path.isfile("../model/t2/sources/%s"%name):
		with open("../model/t2/sources/%s"%name) as file:
			isfile=True
			for cnt,line in enumerate(file.readlines()[:]):
				if cnt+1<100:
					string+="%s"%line
				else:
					print("Values above E%s(100) will not be printed, %s is not included"%(name,line))
	else:
		print("Error message: the %s file on ../model/t2/sources/%s is not prepared"%(name,name))
		isfile=False
	string+="\n"	
	return string,isfile

def diff_seconds(ref,date):
	#Both should be datetime
	if isinstance(date, datetime) and isinstance(ref, datetime):
		diff_seconds=(date-ref).total_seconds()
	else:
		sys.exit("ref or date values are not datetime type")
	return diff_seconds

def PARAM_writer():
	string="PARAM----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"
	params_levels=[1,2,3,4]
	for level in params_levels:
		for key in formats_t2['PARAMETERS']:
			if formats_t2['PARAMETERS'][key][2]==level:
				if key=='RE1' and 'DELTEN' in input_data['PARAMETERS'] and input_data['PARAMETERS']['DELTEN']<0 :
					for index, time_step in enumerate(input_data['PARAMETERS']['DELTEN_LIST']):
						string+=converter(time_step,formats_t2['DELTEN_LIST_FORMAT'][1],False)
						if (index+1)%8==0:
							string+='\n'
					string+='\n'

				value,def_value=def_value_selector('PARAMETERS',key)

				if not isinstance(value,list):
					string+=converter(value,formats_t2['PARAMETERS'][key][1],def_value)
				else:
					for cnt, var in enumerate(value):
						string+=converter(var,formats_t2['PARAMETERS'][key][cnt][2],def_value)
		string+='\n'
	string+="\n"
	return string

def TIMES_writer(input_dictionary=input_data):
	string="TIMES----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"

	if any(input_dictionary['TIMES']['TIMES_N']):
		len_times=len(input_dictionary['TIMES']['TIMES_N'])
		input_dictionary['TIMES']['ITI']=len_times
		input_dictionary['TIMES']['ITE']=len_times
		if len_times>100:
			sys.exit("ITI cannot be higher than 100, reduce the number of times")
	for key in formats_t2['TIMES']:

		value,def_value=def_value_selector('TIMES',key,input_dictionary)

		if not isinstance(value,np.ndarray):
			string+=converter(value,formats_t2['TIMES'][key][1],def_value)
		else:
			string+='\n'
			for index, var in enumerate(value):
				string+=converter(diff_seconds(input_dictionary['ref_date'],var),formats_t2['TIMES']['TIMES_N'][1],False)
				if (index+1)%8==0:
					string+='\n'
	string+="\n\n"
	return string

def source_to_FOFT(db_path=input_data['db_path']):
	"""Crea la seccion FOFT a partir de las fuentes en los pozos

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf

	Returns
	-------
	file
	  FOFT: en ../model/t2/sources
	  
	Examples
	--------
	>>> write_well_sources_from_sqlite_to_FOFT(db_path)
	"""

	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	data_blockcorr=pd.read_sql_query("SELECT blockcorr FROM t2wellsource WHERE source_nickname LIKE 'SRC%';",conn)

	string=""
	for n in sorted(data_blockcorr['blockcorr'].values):
		string+="%s\n"%(n)

	conn.close()
	file=open("../model/t2/sources/FOFT",'w')
	file.write(string)
	file.close()

def wells_track_blocks_to_FOFT(db_path=input_data['db_path']):
	"""Crea la seccion FOFT a partir de la trayectoria del pozo

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf

	Returns
	-------
	file
	  FOFT: en ../model/t2/sources
	  
	Attention
	---------
	En general esta funcion contendra mas de 100 elementos por lo que para la version 6.x de TOUGH2 se recomienda utilizar la funcion write_well_sources_from_sqlite_to_FOFT

	Examples
	--------
	>>> write_wells_track_blocks_from_sqlite_to_FOFT(db_path)
	"""

	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	data_layer=pd.read_sql_query("SELECT correlative FROM layers ORDER BY middle;",conn)

	string=""
	for name in sorted(wells):
		data_block=pd.read_sql_query("SELECT blockcorr FROM t2wellblock  WHERE well='%s' ORDER BY blockcorr;"%name,conn)
		if len(data_block)>0:
			for n in sorted(data_layer['correlative'].values):
				string+="%s\n"%(n+data_block['blockcorr'].values[0])
	conn.close()
	file=open("../model/t2/sources/FOFT",'w')
	file.write(string)
	file.close()

def FOFT_writer():
	string,isfile=FCG_file_checker('FOFT')
	if isfile:
		output="""FOFT ----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"""
		output+=string
	else:
		output=""
	return output

def COFT_writer():
	string,isfile=FCG_file_checker('COFT')
	if isfile:
		output="""COFT ----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"""
		output+=string
	else:
		output=""
	return output

def source_to_GOFT(db_path=input_data['db_path']):
	"""Crea la seccion FOFT a partir de las fuentes en los pozos

	Parameters
	----------
	db_path : str
	  Direccion de base de datos sqlite, tomado de model_conf

	Returns
	-------
	file
	  FOFT: en ../model/t2/sources
	  
	Examples
	--------
	>>> write_well_sources_from_sqlite_to_FOFT(db_path)
	"""

	conn=sqlite3.connect(db_path)
	c=conn.cursor()

	data_blockcorr=pd.read_sql_query("SELECT source_nickname FROM t2wellsource WHERE source_nickname LIKE 'SRC%';",conn)

	string=""
	for n in sorted(data_blockcorr['source_nickname'].values):
		string+="%s\n"%(n)

	conn.close()
	file=open("../model/t2/sources/GOFT",'w')
	file.write(string)
	file.close()

def GOFT_writer():
	string,isfile=FCG_file_checker('GOFT')
	if isfile:
		output="""GOFT ----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"""
		output+=string
	else:
		output=""
	return output

def SOLVR_writer():
	string="SOLVR----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"
	
	cnt=0
	for key in formats_t2['SOLVR']:
		value,def_value=def_value_selector('SOLVR',key)
		if def_value: cnt+=1 
		string+=converter(value,formats_t2['SOLVR'][key][1],def_value)
	string+="\n\n"

	if cnt==len(formats_t2['SOLVR']):
		string=""
	return string

def GENER_adder(type_run=input_data['TYPE_RUN']):
	string="GENER----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"

	if os.path.isfile("../model/t2/sources/GENER_SOURCES") and type_run in ['natural','production']:
		with open("../model/t2/sources/GENER_SOURCES") as gener_source_file:
			for line in gener_source_file.readlines()[:]:
				string+="%s"%line
		if type_run=='natural':
			string+="\n"
		elif type_run=='production':
			if os.path.isfile("../model/t2/sources/GENER_PROD"):
				with open("../model/t2/sources/GENER_PROD") as gener_prod_file:
					for line in gener_prod_file.readlines()[:]:
						string+="%s"%line
			else:
				sys.exit("""Error message: the GENER_PROD file on ../model/t2/sources/GENER_PROD is not prepared\n
					        you might want to run write_gener_from_sqlite on t2gener.py file""")
			
			if os.path.isfile("../model/t2/sources/GENER_MAKEUP"):
				with open("../model/t2/sources/GENER_MAKEUP") as gener_prod_file:
					for line in gener_prod_file.readlines()[:]:
						string+="%s"%line
				string+="\n"
			else:
				print("""Error message: the GENER_MAKEUP file on ../model/t2/sources/GENER_MAKEUP is not prepared\n
					        you might want to run write_gener_from_sqlite on t2gener.py file""")
	else:
		sys.exit("""Error message: the GENER_SOURCES file on ..\model\t2\sources\GENER_SOURCES is not prepared\n
			        you might want to run write_geners_to_txt_and_sqlite on t2gener.py file or the type_run is not correct (natural or production)""")

	return string

def RPCAP_writer():
	string="RPCAP----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"
	params_levels=[1,2]
	for level in params_levels:
		for key in formats_t2['RPCAP']:
			if formats_t2['RPCAP'][key][2]==level:
				value,def_value=def_value_selector('RPCAP',key)
				string+=converter(value,formats_t2['RPCAP'][key][1],def_value)
		string+='\n'
	string+="\n"
	return string

def MULTI_writer():
	string="MULTI----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"
	cnt=0
	for key in formats_t2['MULTI']:
		value,def_value=def_value_selector('MULTI',key)
		if def_value: cnt+=1 
		string+=converter(value,formats_t2['MULTI'][key][1],def_value)
	string+="\n\n"

	if cnt==len(formats_t2['MULTI']):
		string=""
	return string

def TITLE_writer():
	try:
		value=input_data['TITLE']
	except KeyError:
		value=formats_t2['TITLE'][1]
	return  value[0:80].ljust(80) +'\n'

def CONNE_from_steinar_to_t2():
	"""Convierte archivo conne de salida de steinar a archivo CONNE en ../model/t2/sources

	Returns
	-------
	file
	  CONNE: conexiones de elementos
	  
	Attention
	---------
	Existen dos diferencias principales, a primera es el formato de steinar (ancho de columnas) y la segunda es la componente de la gravedad en steinar es +1 cuando cuando deberia ser -1 (debido al en como se escriben los bloques). Tambien introduce la cabezera del archivo.

	Examples
	--------
	>>> CONNE_from_steinar_to_t2()
	"""
	source_file="../mesh/to_steinar/conne"
	output_file="../model/t2/sources/CONNE"

	string="CONNE----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"
	if os.path.isfile(source_file):
		with open(source_file) as rock_file:
			for line in rock_file.readlines()[1:]:
				linex=line.split()
				if float(linex[8])<=0:
					string+="%10s%20s%10.3E%10.3E%10.3E%10.2f\n"%(linex[0],int(linex[4]),float(linex[5]),float(linex[6]),float(linex[7]),float(linex[8]))
				else:
					string+="%10s%20s%10.3E%10.3E%10.3E%10.2f\n"%(linex[0],int(linex[4]),float(linex[5]),float(linex[6]),float(linex[7]),-1*float(linex[8]))
		string+='\n'
		file=open(output_file,'w')
		file.write(string)
		file.close()
	else:
		sys.exit("The file %s or directory do not exist"%source_file)

def merge_ELEME_and_in_to_t2():
	"""Convierte archivo eleme de salida de steinar a archivo ELEME en ../model/t2/sources

	Returns
	-------
	file
	  ELEME: definicion de los elementos en formato TOUGH2
	  
	Attention
	---------
	El archivo eleme de steinar no contiene xyz por lo que esta funcion introduce las coordenadas al archivo de salida ELEME, asi como la cabezera del mismo.

	Examples
	--------
	>>> conne_from_steinar_to_t2()
	"""
	source_file="../mesh/to_steinar/in"
	eleme_file="../mesh/to_steinar/eleme"
	output_file="../model/t2/sources/ELEME"

	if os.path.isfile(source_file) and os.path.isfile(eleme_file):
		string="ELEME----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"
		data_in=pd.read_csv(source_file,delim_whitespace=True,skiprows=7,header=None,names=['block','li','X','Y','Z','h'])
		data_eleme=pd.read_csv(eleme_file,delim_whitespace=True,skiprows=1,header=None,names=['block','rocktype','vol'])
		data_in.set_index('block')
		data_eleme.set_index('block')

		for n in range(len(data_eleme['block'])):
			selection=data_in.loc[data_in['block']==data_eleme['block'][n]].values.tolist()
			if data_eleme['vol'][n]>0:
				string+="%5s%10s%5s%10.3E%10s%10s%10.3E%10.3E%10.3E\n"%(data_eleme['block'][n],\
	                                                                      " ",\
	                                                data_eleme['rocktype'][n],\
	                                                     data_eleme['vol'][n],\
	                                                                       " ",\
	                                                                       " ",\
	                                                           selection[0][2],\
	                                                           selection[0][3],\
	                                                           selection[0][4])

			else:
				string+="%5s%10s%5s%10s%10s%10.3E%10.3E%10.3E\n"%(data_eleme['block'][n],\
					                                                                      " ",\
					                                                data_eleme['rocktype'][n],\
					                                                                       " ",\
					                                                                       " ",\
					                                                           selection[0][2],\
					                                                           selection[0][3],\
					                                                           selection[0][4])

		string+='\n'
		file=open(output_file,'w')
		file.write(string)
		file.close()
	else:
		sys.exit("The file %s or directory do not exist"%file_source_file)

def ELEME_adder():

	source_file="../model/t2/sources/ELEME"
	string=""
	if os.path.isfile(source_file):
		with open(source_file) as eleme_source_file:
			for line in eleme_source_file.readlines()[:]:
				string+="%s"%line
	else:
		sys.exit("The file %s or directory do not exist"%source_file)

	return string

def CONNE_adder():
	source_file="../model/t2/sources/CONNE"
	string=""
	if os.path.isfile(source_file):
		with open(source_file) as conne_source_file:
			for line in conne_source_file.readlines()[:]:
				string+="%s"%line
	else:
		sys.exit("The file %s or directory do not exist"%source_file)
	return string

def t2_input(include_FOFT,include_SOLVR,include_COFT,include_GOFT,include_RPCAP,include_MULTI,include_START,type_run=input_data['TYPE_RUN']):

	secctions=[TITLE_writer,PARAM_writer,TIMES_writer,ELEME_adder,CONNE_adder]

	if_functions_dictionary={'RPCAP':[include_RPCAP,RPCAP_writer],
							 'MULTI':[include_MULTI,MULTI_writer],
							 'FOFT':[include_FOFT,FOFT_writer],
							 'SOLVR':[include_SOLVR,SOLVR_writer],
							 'COFT':[include_COFT,COFT_writer],
							 'GOFT':[include_GOFT,GOFT_writer]}

	for bools in if_functions_dictionary:
		if if_functions_dictionary[bools][0]:
			secctions.append(if_functions_dictionary[bools][1])

	output=""
	for func in secctions:
		if func==PARAM_writer and include_START:
			output+="START----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"
		output+=func()

	output+=GENER_adder(type_run)

	output+="ENDCY----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8\n"

	input_fi_file="../model/t2/t2"
	t2_file_out=open(input_fi_file, "w")
	t2_file_out.write(output)
	t2_file_out.close()	

#t2_input(include_FOFT=True,include_RPCAP=True,include_SOLVR=True,include_COFT=True,include_GOFT=True,include_MULTI=True,include_START=True,type_run='production')
